/*
 * JCards
 * Copyright (C) 2002-2011 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.io.File;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JTable;
import javax.swing.UIManager;
import nrs.jcards.gui.I18NIcon;
import nrs.jcards.gui.action.JCardsAction;

/**
 * User settings
 * 
 * @author Copyright &#169; 2002-2011 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class UserSettings
{
	private final static Logger log = Logger.getLogger(UserSettings.class.getName());
	
	public final static String[] DATE_FORMATS = {
		"dd/MM/yyyy", "dd/MM/yyyy HH:mm", "dd/MM/yy", "dd/MM/yy HH:mm",
		"MM/dd/yyyy", "MM/dd/yyyy HH:mm", "MM/dd/yy", "MM/dd/yy HH:mm",
		"dd.MM.yyyy", "dd.MM.yyyy HH:mm", "MM.yyyy", "yyyy",
		"dd MMM yyyy", "dd MMM yyyy HH:mm", "HH:mm", "HH:mm:ss",
		"yyyy-MM-dd", "yyyy-MM-dd HH:mm", "MM/dd", "MM/yy", "MM/yyyy"
	};
	
	/** Icons */
	public final static String ICON = "/pics/jcards.png";
	public final static String ICON_ADD = "/pics/add.png";
	public final static String ICON_ARROW_DOWN = "/pics/arrow_down.png";
	public final static String ICON_ARROW_RIGHT = "/pics/arrow_right.png";
	public final static String ICON_BACKGROUND = "/pics/jcards-background.png";
	public final static String ICON_CALCULATION = "/pics/calculation.png"; // used in edit card dialog
	public final static String ICON_CANCEL = "/pics/cancel.png";
	public final static String ICON_CATEGORIES = "/pics/categories.png";
	public final static String ICON_CLIPBOARD = "/pics/clipboard.png";
	public final static String ICON_COLUMNS = "/pics/columns.png";
	public static final String ICON_COMMENT = "/pics/comment.png";
	public final static String ICON_COPY = "/pics/copy.png";
	public final static String ICON_COUNTER = "/pics/counter.png";
	public final static String ICON_DB = "/pics/database.png";
	public final static String ICON_DELETE = "/pics/remove.png";
	public final static String ICON_DOWN = "/pics/down.png";
	public final static String ICON_EDIT = "/pics/edit.png";
	public final static String ICON_END = "/pics/end.png";
	public final static String ICON_EXIT = "/pics/exit.png";
	public static final String ICON_EXPORT = "/pics/export.png";
	public final static String ICON_FIELDS = "/pics/fields.png";
	public final static String ICON_FIND = "/pics/find.png";
	public final static String ICON_HELP = "/pics/help.png";
	public static final String ICON_IMAGE = "/pics/image.png";
	public static final String ICON_IMPORT = "/pics/import.png";
	public static final String ICON_ICONS = "/pics/icons.png";
	public final static String ICON_LEFT = "/pics/previous.png";
	public final static String ICON_NEW = "/pics/new.png";
	public final static String ICON_NEXT = "/pics/next.png";
	public final static String ICON_OK = "/pics/ok.png";
	public final static String ICON_OPEN = "/pics/open.png";
	public final static String ICON_OPEN_URL = "/pics/www.png";
	public final static String ICON_PASSWORD = "/pics/password.png";
	public final static String ICON_PREFERENCES = "/pics/configure.png";
	public final static String ICON_PREVIOUS = "/pics/previous.png";
	public final static String ICON_PRINT = "/pics/print.png";
	public final static String ICON_RESET = "/pics/reset.png";
	public final static String ICON_RIGHT = "/pics/next.png";
	public final static String ICON_SAVE = "/pics/save.png";
	public final static String ICON_SAVE_AS = "/pics/saveas.png";
	public final static String ICON_SORT_ASC = "/pics/sort_asc.png";
	public final static String ICON_SORT_DSC = "/pics/sort_dsc.png";
	public final static String ICON_START = "/pics/start.png";
	public final static String ICON_STATS = "/pics/stats.png";
	public final static String ICON_STATS_FIELD = "/pics/stats_field.png";
	public final static String ICON_UNDELETE = "/pics/undelete.png";
	public final static String ICON_UP = "/pics/up.png";
	
	private final static String[][] ALL_ICONS = {
		{ "new", ICON_NEW },
		{ "exit", ICON_EXIT },
		{ "card.del", ICON_DELETE },
		{ "card.edit", ICON_EDIT },
		{ "card.new", ICON_ADD },
		{ "card.undel", ICON_UNDELETE },
		{ "card.copy", ICON_COPY },
		{ "copy.clip", ICON_CLIPBOARD },
		{ "fields", ICON_DB },
		{ "find", ICON_FIND },
		{ "open", ICON_OPEN },
		{ "open.url", ICON_OPEN_URL },
		{ "prefs", ICON_PREFERENCES },
		{ "print", ICON_PRINT },
		{ "save", ICON_SAVE },
		{ "save.as", ICON_SAVE_AS },
		{ "pass.set", ICON_PASSWORD }
	};
	
	private final static String[] DESKTOP_ICONS = {
		"new", "open", "save", "card.new", "card.edit",
		"card.del", "find", "fields", "prefs"
	};
	
	/** Preference names */
	private final static String PREF_AUTO_EDIT = "autoEdit";
	private final static String PREF_BROWSER = "webBrowser";
	private final static String PREF_FONT = "font";
	public final static String PREF_ICONS = "icons";
	public final static String PREF_FILE = "jca.properties";
	public final static String PREF_FILE_DIR = ".jca";
	public final static String PREF_FILE_PATH = PREF_FILE_DIR+"/"+PREF_FILE;
	public static final String PREF_IMAGE_EDITOR = "imageEditor";
	public final static String PREF_LASTFILE = "lastFile";
	public final static String PREF_LOCATION = "location";
	private final static String PREF_OPEN_LAST_FILE = "openLastFile";
	public final static String PREF_SIZE = "size";
	private final static String PREF_SYS_BROWSER = "systemBrowser";
	private final static String PREF_TABLE_EDIT = "tableEdit";
	public static final String PREF_TABLE_RESIZE = "tableResize";
	private static final String PREF_THEME = "theme";
	private final static String PREF_TOOLBAR = "toolbar";
	private final static String PREF_WELCOME = "welcome";
	private final static String PREF_WORD_WRAP = "wordWrap";
	
	/** Last opened files */
	public final static int MAX_RECENT_FILES = 5;
	private List<String> recentFiles = new ArrayList<String>();
	
	/** Environment settings */
	private Dimension overrideSize;
	private Point initLocation;
	private int tableAutoResizeMode = JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS;
	boolean viewToolbar = true;
	private List<I18NIcon> userIcons = null;
	private String browserLocation;
	private String imageEditor;
	private Font font = new Font("SansSerif", Font.PLAIN, 12);
	private String theme = "javax.swing.plaf.metal.MetalLookAndFeel";

	/** Preferences */
	private boolean openLastFile = true;
	private boolean autoEdit = true;
	private boolean wordWrap = true;
	private boolean showWelcome = true;
	private boolean tableEdit = true;
	private boolean useSystemBrowser = true;
	
	/** Creates a new instance of UserSettings */
	public UserSettings()
	{
		// setup initial icons
		resetIcons();
	}
	
	/**
	 * Setup a hashmap of names to icons for all icons
	 * @param keyOnName if true the Map key is the localised name, if false the key
	 * @return a Map of icon mappings
	 * @see I18NIcon
	 */
	public Map<String, I18NIcon> getAllIcons(boolean keyOnName)
	{
		log.entering("UserSettings", "getAllIcons");
		
		// convert to I18NIcons
		// icons equate to menu items which is why they are not redefined in the buttons bundle
		Map<String, I18NIcon> icons = new HashMap<String, I18NIcon>(ALL_ICONS.length);
		I18NIcon icon;
		for (int i=0; i<ALL_ICONS.length; i++)
		{
			log.log(Level.FINEST, "Creating icon {0}", ALL_ICONS[i][0]);
			icon = new I18NIcon();
			icon.setKey(ALL_ICONS[i][0]);
			icon.setName(JCardsAction.getTextWithoutMnemonic(JCards.getMenuString(ALL_ICONS[i][0])));
			icon.setGraphic(ALL_ICONS[i][1]);
			if (keyOnName)
			{
				icons.put(icon.getName(), icon);
			}
			else
			{
				icons.put(icon.getKey(), icon);
			}
		}
		
		log.exiting("UserSettings", "getAllIcons");
		return icons;
	}
	
	/**
	 * Set the user icons
	 * @param keys array of icon key names
	 */
	public final void resetIcons()
	{
		log.entering("UserSettings", "resetIcons");
		
		I18NIcon icon;
		userIcons = new ArrayList<I18NIcon>(DESKTOP_ICONS.length);
		Map<String, I18NIcon> allIcons = getAllIcons(false);
		
		for (String key : DESKTOP_ICONS)
		{
			icon = allIcons.get(key);
			if (icon != null)
			{
				userIcons.add(icon);
			}
		}
		
		log.exiting("UserSettings", "resetIcons");
	}

	/**
	 * Get the preferred frame size
	 * @return Dimension or null
	 */
	public Dimension getOverrideSize()
	{
		return overrideSize;
	}

	public void setOverrideSize(Dimension overrideSize)
	{
		this.overrideSize = overrideSize;
	}

	/**
	 * Get the location for the frame
	 * @return Point or null if not set
	 */
	public Point getInitLocation()
	{
		return initLocation;
	}
	
	public void setInitLocation(Point initLocation)
	{
		this.initLocation = initLocation;
	}
	
	/** Get the icons currently selected by the user */
	public List<I18NIcon> getUserIcons()
	{
		return new ArrayList<I18NIcon>(userIcons);
	}
	
	/**
	 * Return a list of user icons
	 * @return list of comma separated user icon keys
	 */
	public String getUserIconList()
	{
		log.entering("UserSettings", "getUserIconList");
		
		StringBuilder sb = new StringBuilder();
		
		for (I18NIcon icon : userIcons)
		{
			sb.append(icon.getKey());
			sb.append(",");
		}
		
		if (sb.length() > 1) {
			sb.setLength(sb.length()-1); // remove last ,
		}
		
		log.exiting("UserSettings", "getUserIconList", sb.length());
		
		return sb.toString();
	}
	
	/**
	 * Set the icons that the user wants to see keyed on name
	 * @param newUserIcons array of menu keys
	 */
	public void setUserIconKeys(String[] newUserIcons)
	{
		log.entering("UserSettings", "setUserIconKeys", newUserIcons);
		Map<String, I18NIcon> icons = getAllIcons(false);
		setUserIcons(icons, newUserIcons);
		log.exiting("UserSettings", "setUserIconKeys");
	}
	
	/**
	 * Set the user selected icons
	 * @param newUserIcons
	 */
	public void setUserIcons(List<I18NIcon> newUserIcons)
	{
		// copy into new array to prevent inadvertent copying
		userIcons = new ArrayList<I18NIcon>(newUserIcons);
	}
	
	/**
	 * Set the icons that the user wants to see
	 * @param allIcons all icons
	 * @param newUserIcons array of localised menu/icon names
	 */
	private void setUserIcons(Map<String, I18NIcon> allIcons, String[] newUserIcons)
	{
		log.entering("UserSettings", "setUserIcons");
		
		if (newUserIcons.length != 0)
		{
			userIcons = new ArrayList<I18NIcon>(newUserIcons.length);
			I18NIcon icon;
			
			for (String name : newUserIcons)
			{
				icon = allIcons.get(name);
				assert icon != null : "Cannot get icon for "+name;
				if (icon != null)
				{
					userIcons.add(icon);
				}
			}
		}
		log.exiting("UserSettings", "setUserIcons");
	}
	
	/** Returns true if open last file is turned on
	 * @return true if open last file is turned on
	 * @see #setOpenLastFile(boolean)
	 */
	public boolean isOpenLastFile()
	{
		return openLastFile;
	}
	
	/** Set whether to open the last accessed file
	 * @param openLast set to true to open the last accessed file
	 * @see #isOpenLastFile()
	 */
	public void setOpenLastFile(boolean openLast)
	{
		openLastFile = openLast;
	}
	
	/** Returns true if cards are automatically edited upon opening
	 * @see #setAutoEdit(boolean)
	 */
	public boolean isAutoEdit()
	{
		return autoEdit;
	}
	
	/** Set whether cards should be automatically in edit mode when opened
	 * @param ae set to true to always edit cards
	 * @see #isAutoEdit()
	 */
	public void setAutoEdit(boolean ae)
	{
		autoEdit = ae;
	}
	
	/** Returns true if word wrap is on
	 ** @see #setWordWrap(boolean)
	 **/
	public boolean isWordWrap()
	{
		return wordWrap;
	}
	
	/** Set word wrap for memo fields
	 * @param ww set to true for word wrap
	 */
	public void setWordWrap(boolean ww)
	{
		wordWrap = ww;
	}
	
	/** Turn on/off the welcome screen on startup
	 * @param showWelcome set to true to show the welcome screen on startup
	 */
	public void setShowWelcome(boolean showWelcome)
	{
		this.showWelcome = showWelcome;
	}
	
	/**
	 * Should the welcome screen be shown
	 * @return true if the welcome screen should be shown
	 */
	public boolean isShowWelcome()
	{
		return showWelcome;
	}
	
	/** Get settings to store in properties file */
	public Properties getProperties()
	{
		log.entering("UserSettings", "getProperties");
		
		Properties props = new Properties();
		
		// add preferences
		props.put(PREF_AUTO_EDIT, Boolean.toString(autoEdit));
		String fontName = font.getName()+","+font.getStyle()+","+font.getSize();
		props.put(PREF_FONT, fontName);
		props.put(PREF_OPEN_LAST_FILE, Boolean.toString(openLastFile));
		props.put(PREF_SYS_BROWSER, Boolean.toString(useSystemBrowser));
		props.put(PREF_TABLE_EDIT, Boolean.toString(tableEdit));
		props.put(PREF_THEME, theme);
		props.put(PREF_TOOLBAR, Boolean.toString(viewToolbar));
		props.put(PREF_WELCOME, Boolean.toString(showWelcome));
		props.put(PREF_WORD_WRAP, Boolean.toString(wordWrap));
		if (browserLocation != null) {
			props.put(PREF_BROWSER, browserLocation);
		}
		if (imageEditor != null) {
			props.put(PREF_IMAGE_EDITOR, imageEditor);
		}
		
		// add last accessed files
		for (int i=0; i<recentFiles.size(); i++)
		{
			props.put("lastFile"+i, recentFiles.get(i));
		}
		
		log.exiting("UserSettings", "getProperties", props);
		return props;
	}
	
	/**
	 * Return the preferred font if set
	 * @param props Properties
	 */
	public void setProperties(Properties props)
	{
		log.entering("UserSettings", "setProperties", props);
		
		// read preferences
		openLastFile = Boolean.valueOf(props.getProperty(PREF_OPEN_LAST_FILE));
		log.log(Level.CONFIG, "Open last file={0}", openLastFile);
		
		autoEdit = Boolean.valueOf(props.getProperty(PREF_AUTO_EDIT));
		log.log(Level.CONFIG, "Auto edit={0}", autoEdit);
		
		tableEdit = Boolean.valueOf(props.getProperty(PREF_TABLE_EDIT));
		log.log(Level.CONFIG, "Table edit={0}", tableEdit);
		
		wordWrap = Boolean.valueOf(props.getProperty(PREF_WORD_WRAP));
		log.log(Level.CONFIG, "Word wrap={0}", wordWrap);
		
		useSystemBrowser = Boolean.valueOf(props.getProperty(PREF_SYS_BROWSER));
		log.log(Level.CONFIG, "Use system browser={0}", useSystemBrowser);
		
		browserLocation = props.getProperty(PREF_BROWSER);
		log.log(Level.CONFIG, "Browser={0}", browserLocation);

		imageEditor = props.getProperty(PREF_IMAGE_EDITOR);
		log.log(Level.CONFIG, "Image editor={0}", imageEditor);
		
		showWelcome = Boolean.valueOf(props.getProperty(PREF_WELCOME));
		log.log(Level.CONFIG, "Show welcome={0}", showWelcome);
		
		Font pfont = null;
		String fontString = props.getProperty(PREF_FONT);
		if (fontString != null)
		{
			String[] fontParts = fontString.split(",");
			if (fontParts.length == 3)
			{
				try
				{
					pfont = new Font(fontParts[0], Integer.parseInt(fontParts[1]), Integer.parseInt(fontParts[2]));
					font = pfont;
				}
				catch (NumberFormatException nfex)
				{
					log.log(Level.WARNING, "Bad font: {0}", fontString);
				}
			}
		}
		log.log(Level.CONFIG, "Font={0}", font);
		
		viewToolbar = Boolean.valueOf(props.getProperty(PREF_TOOLBAR, "true"));
		log.log(Level.CONFIG, "View toolbar={0}", viewToolbar);

		String ptheme = props.getProperty(PREF_THEME);
		log.log(Level.CONFIG, "Theme={0}", ptheme);
		if (ptheme != null) {
			theme = ptheme;
		}
		
		String size = props.getProperty(PREF_SIZE);
		if (size != null)
		{
			log.log(Level.CONFIG, "Size={0}", size);
			try
			{
				int pos = size.indexOf("x");
				if (pos > 0)
				{
					String w = size.substring(0, pos);
					String h = size.substring(pos+1);
					setOverrideSize(new Dimension(Integer.parseInt(w), Integer.parseInt(h)));
				}
			}
			catch (Exception ex)
			{
				log.log(Level.CONFIG, "setProperties: Cannot set size", ex);
			}
		}
		
		String location = props.getProperty(PREF_LOCATION);
		if (location != null)
		{
			log.log(Level.CONFIG, "Location={0}", location);
			try
			{
				int pos = location.indexOf(",");
				if (pos > 0)
				{
					String x = location.substring(0, pos);
					String y = location.substring(pos+1);
					setInitLocation(new Point(Integer.parseInt(x), Integer.parseInt(y)));
				}
			}
			catch (Exception ex)
			{
				log.log(Level.CONFIG, "setProperties: Cannot set initial location", ex);
			}
		}

		String trMode = props.getProperty(PREF_TABLE_RESIZE);
		if (trMode != null)
		{
			log.log(Level.CONFIG, "Table reside mode={0}", trMode);
			try {
				tableAutoResizeMode = Integer.parseInt(trMode);
			} catch (NumberFormatException nfex) {
				log.log(Level.CONFIG, "setProperties: Cannot set table resize mode", nfex);
			}
		}
		
		// read icons
		String icons = props.getProperty(PREF_ICONS);
		if (icons != null)
		{
			log.log(Level.CONFIG, "Icons={0}", icons);
			String[] iconArray = icons.split(",");
			setUserIconKeys(iconArray);
		}
		
		recentFiles.clear();
		
		// read recent files
		{
			String recentFile;
			File f;
			String path;
			for (int i=0; i<UserSettings.MAX_RECENT_FILES; i++)
			{
				recentFile = props.getProperty(UserSettings.PREF_LASTFILE+i);
				if (recentFile != null)
				{
					f = new File(recentFile);
					path = f.getAbsolutePath();
					if (f.exists() && f.isFile() && !recentFiles.contains(path))
					{
						recentFiles.add(path);
						log.log(Level.CONFIG, "Recent file {0}", path);
					}
				}
			}
		}
		
		log.exiting("UserSettings", "setProperties");
	}
	
	/** Is toolbar visible */
	public boolean isViewToolbar()
	{
		return viewToolbar;
	}
	
	public void setViewToolbar(boolean vToolbar)
	{
		viewToolbar = vToolbar;
	}
	
	public List<String> getRecentFiles()
	{
		return recentFiles;
	}
	
	/** Add a recent file */
	public void addRecentFile(String file)
	{
		log.entering("UserSettings", "addRecentFile", file);
		
		recentFiles.remove(file);
		recentFiles.add(0, file);
		if (recentFiles.size() > MAX_RECENT_FILES)
		{
			recentFiles.remove(MAX_RECENT_FILES);
		}
		
		log.exiting("UserSettings", "addRecentFile");
	}
	
	/**
	 * Get the location of the web browser
	 * @return web browser location
	 */
	public String getBrowserLocation()
	{
		if (browserLocation == null)
		{
			List<String> bList = findWebBrowser();
			if (!bList.isEmpty())
			{
				browserLocation = bList.get(0);
			}
		}
		
		return browserLocation;
	}
	
	public void setBrowserLocation(String browserLocation)
	{
		this.browserLocation = browserLocation;
	}
	
	/**
	 * Look through usual places and see if web browser can be identified
	 * @return list of paths to found web browsers - may be empty
	 */
	public static List<String> findWebBrowser()
	{
		ResourceBundle browserBundle = ResourceBundle.getBundle("nrs.jcards.props.browsers");
		String os = System.getProperty("os.name").toLowerCase();
		List<String> browserList = new ArrayList<String>();
		File f;
		
		if (os.contains("mac"))
		{
			browserList.add("safari");
		}
		else if (os.contains("win"))
		{
			String[] winBrowsers = browserBundle.getString("win.browsers").split(",");
			for (String winBrowser : winBrowsers)
			{
				f = new File("C:/Program Files/", winBrowser);
				if (f.exists() && f.isFile() && !browserList.contains(f.getAbsolutePath()))
				{
					browserList.add(f.getAbsolutePath());
				}
			}
		}
		else // assume unix
		{
			String[] unixPaths = browserBundle.getString("unix.paths").split(",");
			for (String path : unixPaths)
			{
				String[] browsers = browserBundle.getString("unix.browsers").split(",");
				for (String browser : browsers)
				{
					f = new File(path, browser);
					if (f.exists() && f.isFile() && !browserList.contains(f.getAbsolutePath()))
					{
						browserList.add(f.getAbsolutePath());
					}
				}
			}
		}

		Collections.sort(browserList);
		
		return browserList;
	}

	/**
	 * Get the user's preferred font
	 * @return font
	 */
	public Font getFont()
	{
		return font;
	}

	/**
	 * Set the user's preferred font
	 * @param font
	 */
	public void setFont(Font font)
	{
		if (font != null) {
			this.font = font;
		}
	}

	/**
	 * Get the theme in use
	 * @return class name of the theme in use
	 */
	public String getTheme()
	{
		return theme;
	}

	/**
	 * Set the theme and update the UIManager
	 * @param newTheme class name of the theme to use
	 */
	public void setTheme(String newTheme)
	{
		if (newTheme != null)
		{
			try
			{
				UIManager.setLookAndFeel(newTheme);
				theme = newTheme;
			} catch (Exception ex) {
				log.log(Level.WARNING, "Cannot set theme to "+newTheme, ex);
			}
		}
	}

	/**
	 * Is table editing turned on or off
	 * @return true to allow table editing, false for read-only
	 */
	public boolean isTableEdit()
	{
		return tableEdit;
	}

	/**
	 * Sets table editing on or off
	 * @param tableEdit
	 */
	public void setTableEdit(boolean tableEdit)
	{
		this.tableEdit = tableEdit;
	}

	public boolean isUseSystemBrowser()
	{
		return useSystemBrowser;
	}

	public void setUseSystemBrowser(boolean useSystemBrowser)
	{
		this.useSystemBrowser = useSystemBrowser;
	}

	/**
	 * Get the table resize mode that may have been set previously.
	 * Should only be used during initialisation
	 * @return
	 */
	public int getTableAutoResizeMode()
	{
		return tableAutoResizeMode;
	}

	/**
	 * @return the imageEditor
	 */
	public String getImageEditor()
	{
		return imageEditor;
	}

	/**
	 * @param imageEditor the imageEditor to set
	 */
	public void setImageEditor(String imageEditor)
	{
		this.imageEditor = imageEditor;
	}
}
