/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.fields;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import nrs.jcards.JCards;
import nrs.jcards.gui.JCardsFrame;

/**
 * Boolean field type
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class BooleanFieldDef extends FieldDef
{
	public static String YES = "Y";
	public static String NO = "N";
	
	static
	{
		YES = JCards.getLabelString("initial.yes");
		NO = JCards.getLabelString("initial.no");
	}
	
	/** Creates a new instance of BooleanFieldDef */
	public BooleanFieldDef(String name)
	{
		setName(name);
		type = FieldType.BOOLEAN;
	}
	
	/** Set the default value of this boolean field */
	@Override public boolean setDefaultValue(String value)
	{
		boolean set = false;
		
		if (value != null)
		{
			String uc = value.toUpperCase();
			if (uc.equalsIgnoreCase(YES) || uc.equalsIgnoreCase("Y") || uc.equals("1") || uc.equalsIgnoreCase("true"))
			{
				defaultValue = YES;
				set = true;
			}
			else if (uc.equalsIgnoreCase(NO) || uc.equalsIgnoreCase("N") || uc.equals("0") || uc.equalsIgnoreCase("false"))
			{
				defaultValue = NO;
				set = true;
			}
		}
		
		return set;
	}
	
	@Override
	public String formatValue(Object obj)
	{
		String formattedVal = null;
		if (obj != null)
		{
			if (Boolean.parseBoolean(obj.toString())) {
				formattedVal = YES;
			} else {
				formattedVal = NO;
			}
		}
		return formattedVal;
	}
	
	/**
	 * Parse the value and return the correct type
	 * @param value the value to parse
	 * @return the parsed value
	 */
	@Override public Object parseValue(String value)
	{
		boolean parsedVal = false;
		
		if (value != null && (value.equalsIgnoreCase(YES) || value.equalsIgnoreCase("Y") || value.equals("1") || value.equalsIgnoreCase("true")))
		{
			parsedVal = true;
		}
		
		return parsedVal;
	}
	
	@Override public Class getFieldClass()
	{
		return Boolean.class;
	}
	
	@Override public void setGUIValue(JComponent comp, Object value, boolean editing, boolean newCard)
	{
		if (value == null)
		{
			if (newCard && YES.equals(getDefaultValue()))
			{
				value = Boolean.TRUE;
			}
			else
			{
				return;
			}
		}
		
		JCheckBox cb = (JCheckBox)comp;
		Boolean check = (Boolean)value;
		if (check.booleanValue()) {
			cb.setSelected(true);
		}
	}
	
	/**
	 * Get the GUI component that represents the field
	 */
	@Override public JComponent getGUIComponent(boolean newCard, boolean editCard, boolean copyCard, JCardsFrame frame)
	{
		log.entering("BooleanFieldDef", "getGUIComponent");
		
		JCheckBox cb = new JCheckBox();
		cb.setEnabled(editCard);
		cb.setName(name);
		
		log.exiting("BooleanFieldDef", "getGUIComponent");
		
		return cb;
	}
	
	@Override public InputCheck checkInputValue(JComponent comp)
	{
		JCheckBox cb = (JCheckBox)comp;
		return InputCheck.validInputCheck(cb.isSelected());
	}
}
