/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.fields;

import java.util.Map;
import javax.swing.JComponent;
import javax.swing.JLabel;
import nrs.jcards.gui.CalculatedLabel;
import nrs.jcards.gui.JCardsFrame;
import nrs.jcards.io.Elements;

/**
 * Calculated field definition
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CalculatedFieldDef extends NumberFieldDef
{
	/** Creates a new instance of CalculatedFieldDef */
	public CalculatedFieldDef(String name)
	{
		super(name);
		type = FieldType.CALCULATED;
	}
	
	/**
	 * Get the field definition
	 * @return Map of name/value pairs
	 */
	@Override public Map<String, String> getFieldDetails()
	{
		Map<String, String> def = super.getFieldDetails();
		def.put(Elements.CALC, defaultValue);
		def.remove(Elements.DEFAULT);
		
		return def;
	}

	@Override public void setFieldDetails(Map<String, String> fieldDetails)
	{
		log.entering("CalculatedFieldDef", "setFieldDetails", fieldDetails);
		
		super.setFieldDetails(fieldDetails);
		defaultValue = fieldDetails.get(Elements.CALC);
		
		log.exiting("CalculatedFieldDef", "setFieldDetails");
	}
	
	public void setCalculation(String calculation)
	{
		defaultValue = calculation;
	}
	
	@Override public String getCalculation()
	{
		return defaultValue;
	}
	
	/**
	 * Get the GUI component that represents the field
	 */
	@Override public JComponent getGUIComponent(boolean newCard, boolean editCard, boolean copyCard, JCardsFrame frame)
	{
		log.entering("CalculatedFieldDef", "getGUIComponent");
		
		CalculatedLabel calcLabel = new CalculatedLabel();
		calcLabel.setName(name);
		addCopyPopupMenu(calcLabel, editCard);
		addCopyDnD(calcLabel);
		
		log.exiting("CalculatedFieldDef", "getGUIComponent");
			
		return calcLabel;
	}
	
	/**
	 * Set the value of the component programmatically
	 *
	 * @param comp the component to be updated
	 * @param value the value to set
	 * @param editing true if edit component; false for read-only component
	 * @param newCard true if opened a new card
	 */
	@Override public void setGUIValue(JComponent comp, Object value, boolean editing, boolean newCard)
	{
		log.entering("CalculatedFieldDef", "setGUIValue", value);
		
		if (editing)
		{
			CalculatedLabel calcLabel = (CalculatedLabel)comp;
			if (value != null)
			{
				calcLabel.setText(value.toString());
			}
			calcLabel.setValue((Double)value);
		}
		else
		{
			if (value != null)
			{
				JLabel calcLabel = (JLabel)comp;
				Number num = (Number)value;
				calcLabel.setText(numFormat.format(num));
			}
		}
		
		log.exiting("CalculatedFieldDef", "setGUIValue");
	}
	
	/**
	 * Check the value entered by a user. This is different from parseValue in that
	 * it also takes account of the mandatory flag
	 * @param comp the component to be checked
	 * @return InputCheck
	 */
	@Override public InputCheck checkInputValue(JComponent comp)
	{
		log.entering("CalculatedFieldDef", "checkInputValue", comp);

		CalculatedLabel label = (CalculatedLabel)comp;
		String text = label.getText();
		InputCheck ic;
		if (ERROR.equals(text))
		{
			ic = InputCheck.invalidInputCheck(Double.toString(ERROR_NUM));
			comp.requestFocus();
		}
		else
		{
			ic = InputCheck.validInputCheck(label.getValue());
		}
		
		log.exiting("CalculatedFieldDef", "checkInputValue");
		
		return ic;
	}
}
