/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.fields;

import java.util.Map;
import javax.swing.JComponent;
import javax.swing.JTextField;
import nrs.jcards.Counter;
import nrs.jcards.DBDef;
import nrs.jcards.gui.JCardsFrame;
import nrs.jcards.io.Elements;

/**
 * Counter field definition
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CounterFieldDef extends NumberFieldDef
{
	private String countName = null;
	
	public CounterFieldDef(String name)
	{
		super(name);
		type = FieldType.COUNTER;
	}
	
	/** Creates a new instance of CounterFieldDef */
	public CounterFieldDef(String name, String counterName)
	{
		this(name);
		countName = counterName;
	}
	
	@Override public void setFieldDetails(Map<String, String> fieldDetails)
	{
		log.entering("CounterFieldDef", "setFieldDetails");
		
		super.setFieldDetails(fieldDetails);
		countName = fieldDetails.get(Elements.COUNTER);
		
		log.exiting("CounterFieldDef", "setFieldDetails");
	}
	
	/** This field uses a counter to get its values */
	public void setCounter(String newCount)
	{
		countName = newCount;
	}
	
	/**
	 * Get the field's counter
	 * @return the counter or null if not set
	 */
	public String getCounter()
	{
		return countName;
	}
	
	@Override public String toString()
	{
		StringBuilder sb = new StringBuilder(super.toString());
		sb.append(",");
		sb.append(countName);
		return sb.toString();
	}
	
	/**
	 * Get the field definition
	 * @return Map of name/value pairs
	 */
	@Override public Map<String, String> getFieldDetails()
	{
		Map<String, String> def = super.getFieldDetails();
		def.put(Elements.COUNTER, countName);
		
		return def;
	}
	
	/**
	 * Get the GUI component that represents the field
	 */
	@Override public JComponent getGUIComponent(boolean newCard, boolean editCard, boolean copyCard, JCardsFrame frame)
	{
		log.entering("CounterFieldDef", "getGUIComponent");
		
		JComponent comp = super.getGUIComponent(newCard, editCard, copyCard, frame);
		if (newCard || copyCard)
		{
			JTextField tf = (JTextField)comp;
			Counter count = DBDef.getInstance().getCounter(getCounter());
			if (count != null)
			{
				count.increment();
				tf.setText(Double.toString(count.getValue()));
			}
			else
			{
				tf.setText(ERROR);
			}
		}
		
		log.exiting("CounterFieldDef", "getGUIComponent");
		
		return comp;
	}
}
