/*
 * JCards
 * Copyright (C) 2002-2010 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.fields;

/**
 * Field Types
 * 
 * @author Copyright &#169; 2002-2010 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public enum FieldType
{
    BOOLEAN ("Boolean") {
		@Override
	    public FieldDef createFieldDef(String name) { return new BooleanFieldDef(name); }
    },
    CALCULATED ("Calculated") {
		@Override
	    public FieldDef createFieldDef(String name) { return new CalculatedFieldDef(name); }
    },
    CATEGORY ("Category") {
		@Override
	    public FieldDef createFieldDef(String name) { return new CategoryFieldDef(name); }
    },
    COUNTER ("Counter") {
		@Override
	    public FieldDef createFieldDef(String name) { return new CounterFieldDef(name); }
    },
    DATE ("Date") {
		@Override
	    public FieldDef createFieldDef(String name) { return new DateFieldDef(name); }
    },
	DOUBLE ("Double", false)
	{
		@Override
		public FieldDef createFieldDef(String name)	{ return new NumberFieldDef(name); }
	},
    IMAGE ("Image") {
		@Override
		public FieldDef createFieldDef(String name) { return new ImageFieldDef(name); }
    },
	INTEGER ("Integer", false)
	{
		@Override
		public FieldDef createFieldDef(String name)
		{
			NumberFieldDef nfd = new NumberFieldDef(name);
			nfd.setNumType(NumberFieldDef.NUMBER_TYPE.INTEGER);
			return nfd;
		}
	},
	LIST ("List") {
		@Override
	    public FieldDef createFieldDef(String name) { return new ListFieldDef(name); }
    },
    MEMO ("Memo") {
		@Override
	    public FieldDef createFieldDef(String name) { return new MemoFieldDef(name); }
    },
    NUMBER ("Number") {
		@Override
	    public FieldDef createFieldDef(String name) { return new NumberFieldDef(name); }
    },
    STRING ("String") {
		@Override
	    public FieldDef createFieldDef(String name) { return new StringFieldDef(name); }
    };

    private String name;
	private boolean showInFieldList = true;

    FieldType(String name)
    {
	    this.name = name;
    }
	
	FieldType(String name, boolean showInFieldList)
	{
		this.name = name;
		this.showInFieldList = showInFieldList;
	}

    public abstract FieldDef createFieldDef(String name);

    public String getName()
    {
	    return name;
    }
	
	public void setShowInFieldList(boolean show)
	{
		showInFieldList = show;
	}
	
	public boolean isShowInFieldList()
	{
		return showInFieldList;
	}
}
