/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.fields;

import java.util.Locale;

/**
 * Interface for number FieldDefs
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public interface INumberField
{
    /** Number type used for formatting */
    public enum NUMBER_TYPE { NUMBER, INTEGER, CURRENCY, PERCENTAGE };
    
    /**
     * Get the numeric type used for formatting
     * @return NUMBER_TYPE
     */
    public NUMBER_TYPE getNumType();
    
    /**
     * Set the number type
     * @param numType the new NUMBER_TYPE
	 * @param locale the locale which is used when formatting numbers
     */
    public void setNumType(NUMBER_TYPE numType, Locale locale);
    
    /**
     * Set the number of digits after the decimal place
     * @param decimalPlaces number of decimal places to display
     */
    public void setDecimalPlaces(int decimalPlaces);
    
    /**
     * Get the number of displayed decimal places
     * @return number of displayed decimal places
     */
    public int getDecimalPlaces();
}
