/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.fields;

import nrs.jcards.JCards;

/**
 * Represents the result of checking a value
 * 
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class InputCheck
{
	private Object value;
	private boolean valid;
	private String message;

	private InputCheck(boolean valid)
	{
		this.valid = valid;
	}

	/**
	 * Utility method to return an InputCheck advising of a mandatory field
	 * @param name name of the field
	 */
	public static InputCheck mandatoryInputCheck(String name)
	{
		InputCheck ic = new InputCheck(false);
		ic.setMessage(JCards.getFormattedMessage("mandatory.field.msg", name));
		return ic;
	}

	public static InputCheck invalidInputCheck(String message)
	{
		InputCheck ic = new InputCheck(false);
		ic.setMessage(message);
		return ic;
	}

	public static InputCheck validInputCheck(Object value)
	{
		InputCheck ic = new InputCheck(true);
		ic.setValue(value);
		return ic;
	}

	private void setMessage(String message)
	{
		this.message = message;
	}

	private void setValue(Object value)
	{
		this.value = value;
	}

	/**
	 * @return the valid value, may be null if incorrect
	 */
	public Object getValue()
	{
		return value;
	}

	/**
	 * @return is the value valid
	 */
	public boolean isValid()
	{
		return valid;
	}

	/**
	 * @return a failure message, may be null if the value is valid
	 */
	public String getMessage()
	{
		return message;
	}
}
