/*
 * JCards
 * Copyright (C) 2002-2010 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.fields;

import java.util.List;
import java.util.logging.Level;
import javax.swing.*;
import nrs.jcards.DBDef;
import nrs.jcards.gui.JCardsFrame;

/**
 * List field definition
 *
 * @author Copyright &#169; 2002-2010 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class ListFieldDef extends CategoryFieldDef
{
	public ListFieldDef(String name)
	{
		super(name);
		type = FieldType.LIST;
	}

	@Override
	public InputCheck checkInputValue(JComponent comp)
	{
		log.entering("ListFieldDef", "checkInputValue", comp);

		InputCheck ic;
		JList l = (JList)comp;
		Object[] ovals = l.getSelectedValues();
		if (mandatory && (ovals.length == 0))
		{
			l.requestFocus();
			ic = InputCheck.mandatoryInputCheck(name);
		}
		else
		{
			StringBuilder sb = new StringBuilder();
			for (Object o : ovals)
			{
				sb.append(o.toString());
				sb.append(",");
			}
			sb.setLength(sb.length()-1);
			ic = InputCheck.validInputCheck(sb.toString());
		}

		log.exiting("ListFieldDef", "checkInputValue", ic);
		return ic;
	}

	private void addListItems(JList list)
	{
		log.entering("ListFieldDef", "addListItems");

		DefaultListModel dlm = (DefaultListModel)list.getModel();

		// add category item
		List<String> catItems = DBDef.getInstance().getCategory(catName);
		if (catItems != null)
		{
			for (String item : catItems)
			{
				dlm.addElement(item);
			}
		}
		else
		{
			log.log(Level.WARNING, "User has deleted category {0}", name);
		}

		log.exiting("ListFieldDef", "addListItems");
	}

	@Override
	public JComponent getGUIComponent(boolean newCard, boolean editCard, boolean copyCard, JCardsFrame frame)
	{
		JComponent jcomp;
		if (editCard)
		{
			JList list = new JList();
			list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
			list.setName(name);
			list.setModel(new DefaultListModel());
			list.setVisibleRowCount(5);
			addListItems(list);
			JScrollPane sp = new JScrollPane(list);
			jcomp = sp;
		}
		else
		{
			JLabel label = new JLabel();
			label.setName(name);
			addCopyPopupMenu(label, editCard);
			addCopyDnD(label);
			jcomp = label;
		}

		log.exiting("ListFieldDef", "getGUIComponent");
		return jcomp;
	}

	@Override
	public void setGUIValue(JComponent comp, Object value, boolean editing, boolean newCard)
	{
		log.entering("ListFieldDef", "setGUIValue", value);

		if (editing)
		{
			JList list = (JList)comp;
			if (value != null)
			{
				ListSelectionModel lsm = list.getSelectionModel();
				DefaultListModel dlm = (DefaultListModel)list.getModel();
				int index;
				String[] split = value.toString().split(",");
				for (String s : split)
				{
					index = dlm.indexOf(s);
					lsm.addSelectionInterval(index, index);
				}
			}
			else if (newCard && defaultValue != null)
			{
				list.setSelectedValue(defaultValue, true);
			}
		}
		else
		{
			if (value != null)
			{
				JLabel label = (JLabel)comp;
				label.setText(value.toString());
			}
		}

		log.exiting("ListFieldDef", "setGUIValue");
	}

	@Override
	public Class getFieldClass()
	{
		return JList.class;
	}
}
