/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.util.Collections;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import javax.swing.AbstractListModel;
import nrs.jcards.DBDef;
import nrs.jcards.DBDefUpdate;

/**
 * A list model that is aware of category changes within DBDef
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CategoryItemsListModel extends AbstractListModel implements Observer
{
	private String catName;
	private List<String> catItems;
	private DBDef dbDef = DBDef.getInstance();
		
	/** Creates a new instance of CategoriesListModel */
	public CategoryItemsListModel(String catName)
	{
		this.catName = catName;
		catItems = dbDef.getCategory(catName); // local copy for convenience
		dbDef.addObserver(this);
	}
	
	@Override public Object getElementAt(int index)
	{
		return catItems.get(index);
	}
	
	@Override public int getSize()
	{
		return (catItems != null ? catItems.size() : 0);
	}
	
	/** DBDef was updated */
	@SuppressWarnings("unchecked")
	@Override
	public void update(Observable o, Object arg)
	{
		assert arg != null : "CategoryItemsListModel update arg is null";

		DBDefUpdate update = (DBDefUpdate)arg;
		switch (update.getEvent())
		{
			case CATEGORY_ITEM_ADDED:
				fireIntervalAdded(this, catItems.size()-1, catItems.size()-1);
				break;

			case CATEGORY_ITEM_REMOVED:
				fireIntervalRemoved(this, update.getOldIndex(), update.getOldIndex());
				break;
				
			case CATEGORY_ITEM_RENAMED:
				fireContentsChanged(this, update.getNewIndex(), update.getNewIndex());
				break;

			case CATEGORY_UPDATED:
				catName = update.getNewName();
				catItems = (List<String>)update.getNewObject();
				fireContentsChanged(this, 0, update.getNewSize());
				break;
				
			case CATEGORY_ITEM_MOVED:
				fireContentsChanged(this, update.getOldIndex(), update.getNewIndex());
				break;
		}
	}
	
	/** Sort the list in ascending order */
	void sortAscending()
	{
		Collections.sort(catItems);
		dbDef.setCategory(catName, catItems); // required to notify any other observers
	}
	
	/** Sort the list in descending order */
	void sortDescending()
	{
		Collections.sort(catItems);
		Collections.reverse(catItems);
		dbDef.setCategory(catName, catItems); // required to notify any other observers
	}
}
