/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.util.*;
import javax.swing.AbstractListModel;
import nrs.jcards.DBDef;
import nrs.jcards.DBDefUpdate;

/**
 * List model for categories
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CategoryListModel extends AbstractListModel implements Observer
{
	private List<String> catList;
	
	/** Creates a new instance of CategoryListModel */
	public CategoryListModel()
	{
		Map<String, List<String>> catMap = DBDef.getInstance().getCategories();
		catList = new ArrayList<String>(catMap.size());
		catList.addAll(catMap.keySet());
	}
	
	/** DBDef categories changed */
	@Override public void update(Observable o, Object arg)
	{
		assert arg != null : "CategoryListModelupdate arg is null";
		
		DBDefUpdate update = (DBDefUpdate)arg;
		int index;
		switch (update.getEvent())
		{
			case CATEGORY_REMOVED:
				index = catList.indexOf(update.getOldName());
				catList.remove(update.getOldName());
				fireIntervalRemoved(this, index, index);
				break;
				
			case CATEGORY_RENAMED:
				index = catList.indexOf(update.getOldName());
				catList.remove(index);
				catList.add(index, update.getNewName());
				fireContentsChanged(this, index, index);
				break;
				
			case CATEGORY_UPDATED:
				String catName = update.getNewName();
				if (!catList.contains(catName))
				{
					catList.add(catName);
					fireIntervalAdded(this, catList.size()-1, catList.size()-1);
				}
				break;
		}
	}
	
	@Override public Object getElementAt(int index)
	{
		return catList.get(index);
	}
	
	/**
	 * Return the number of category names
	 * @return number of categories
	 */
	@Override public int getSize()
	{
		return catList.size();
	}
}
