/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.logging.Logger;
import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import nrs.jcards.DBDef;
import nrs.jcards.JCards;
import nrs.jcards.gui.dialog.DBDialog;

/**
 * Allows the user to edit categories
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CategoryPanel extends JPanel implements DBDialogActionsI, ListSelectionListener
{
	private DBDialog dbDialog;
	private JList catNameList;
	private CategoryListModel catListModel;
	private EditCategoryPanel ecp;
	private static final Logger log = Logger.getLogger(CategoryPanel.class.getName());
	
	/** Creates new CategoryPanel */
	public CategoryPanel(DBDialog dbDialog)
	{
		this.dbDialog = dbDialog;
		
		setLayout(new BorderLayout());
		
		// set up list of categories
		catListModel = new CategoryListModel();
		catNameList = new JList(catListModel);
		catNameList.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		catNameList.addListSelectionListener(this);
		DBDef.getInstance().addObserver(catListModel);
		JScrollPane scroll = new JScrollPane(catNameList);
		add(scroll, BorderLayout.CENTER);
	}
	
	/** User selected a category name */
	@Override
	public void valueChanged(ListSelectionEvent e)
	{
		log.entering("CategoryPanel", "valueChanged");
		String catName = (String)catNameList.getSelectedValue();
		
		if (catName != null)
		{
			ecp = new EditCategoryPanel(catName, dbDialog);
			dbDialog.setContentPanel(ecp);
			ecp.focusCategoryName();
		}
		log.exiting("CategoryPanel", "valueChanged");
	}
	
	/** New category */
	@Override
	public void newItem()
	{
		log.entering("CategoryPanel", "newItem");
		String catName = JOptionPane.showInputDialog(dbDialog, JCards.getLabelString("name"), JCards.getMessage("new.category"), JOptionPane.QUESTION_MESSAGE);
		if (catName != null)
		{
			catName = catName.trim(); // remove spaces
			if (catName.length() > 0)
			{
				DBDef.getInstance().setCategory(catName, new ArrayList<String>());
				catNameList.setSelectedValue(catName, true);
			}
		}
		log.exiting("CategoryPanel", "newItem");
	}
	
	/** Delete category*/
	@Override
	public boolean deleteItem()
	{
		log.entering("CategoryPanel", "deleteItem");
		boolean deleted = false;
		
		String catName = (String)catNameList.getSelectedValue();
		if (catName != null)
		{
			String field = DBDef.getInstance().removeCategory(catName);
			if (field == null) {
				deleted = true;
			}
			else
			{
				JOptionPane.showMessageDialog(dbDialog,
						JCards.getFormattedMessage("category.in.use", field),
						JCards.getMessage("cannot.delete"), JOptionPane.WARNING_MESSAGE);
			}
		}
		
		log.exiting("CategoryPanel", "deleteItem", deleted);
		return deleted;
	}

	@Override
	public boolean isItemSelected()
	{
		return !catNameList.isSelectionEmpty();
	}
	
	
	/** Remove any current selection */
	@Override
	public void clearSelection()
	{
		catNameList.clearSelection();
	}

	@Override
	public void cleanup()
	{
		DBDef.getInstance().deleteObserver(catListModel);
	}
}
