/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.util.*;
import javax.swing.AbstractListModel;
import nrs.jcards.Counter;
import nrs.jcards.DBDef;
import nrs.jcards.DBDefUpdate;

/**
 * List model that monitors DBDef for changes in Counters
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CounterListModel extends AbstractListModel implements Observer
{
	private List<String> countersList;
	
	/** Creates a new instance of CounterListModel */
	public CounterListModel()
	{
		Map<String, Counter> counters = DBDef.getInstance().getCounters();
		countersList = new ArrayList<String>(counters.size());
		countersList.addAll(counters.keySet());
	}
	
	@Override public Object getElementAt(int index)
	{
		return countersList.get(index);
	}
	
	/** Return the number of counters */
	@Override public int getSize()
	{
		return countersList.size();
	}
	
	/** DBDef changed */
	@Override public void update(Observable o, Object arg)
	{
		assert arg != null : "CounterListModel update arg is null";
		
		DBDefUpdate update = (DBDefUpdate)arg;
		int index;
		switch (update.getEvent())
		{
			case COUNTER_ADDED:
				countersList.add(update.getNewName());
				fireIntervalAdded(this, countersList.size()-1, countersList.size()-1);
				break;
				
			case COUNTER_DELETED:
				index = countersList.indexOf(update.getOldName());
				countersList.remove(update.getOldName());
				fireIntervalRemoved(this, index, index);
				break;
				
			case COUNTER_UPDATED:
				if (!update.getOldName().equals(update.getNewName()))
				{
					index = countersList.indexOf(update.getOldName());
					countersList.remove(update.getOldName());
					countersList.add(index, update.getNewName());
					fireContentsChanged(this, index, index);
				}
				break;
		}
	}
}
