/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.awt.BorderLayout;
import java.util.logging.Logger;
import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import nrs.jcards.Counter;
import nrs.jcards.DBDef;
import nrs.jcards.JCards;
import nrs.jcards.gui.dialog.DBDialog;

/**
 * List of Counters
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CounterPanel extends JPanel implements DBDialogActionsI, ListSelectionListener
{
	private DBDialog dbDialog;
	private JList countersList;
	private CounterListModel counterListModel;
	private static final Logger log = Logger.getLogger(CounterPanel.class.getName());
	
	/** Creates a new instance of CounterPanel */
	public CounterPanel(DBDialog dbDialog)
	{
		this.dbDialog = dbDialog;
		
		setLayout(new BorderLayout());
		
		counterListModel = new CounterListModel();
		countersList = new JList(counterListModel);
		countersList.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
		countersList.addListSelectionListener(this);
		DBDef.getInstance().addObserver(counterListModel);
		JScrollPane scroll = new JScrollPane(countersList);
		add(scroll, BorderLayout.CENTER);
	}
	
	/** User selected a different Counter */
	@Override
	public void valueChanged(ListSelectionEvent e)
	{
		log.entering("CounterPanel", "valueChanged");
		
		String counterName = (String)countersList.getSelectedValue();
		if (counterName != null)
		{
			Counter counter = DBDef.getInstance().getCounter(counterName);
			assert counter != null : "CounterPanel.valueChanged cannot get counter for "+counterName;
			EditCounterPanel ecp = new EditCounterPanel(dbDialog, counter);
			dbDialog.setContentPanel(new JScrollPane(ecp));
			ecp.focusCounterName();
		}
		log.exiting("CounterPanel", "valueChanged");
	}
	
	/** Add a new counter */
	@Override
	public void newItem()
	{
		log.entering("CounterPanel", "newItem");
		String newCounter = JOptionPane.showInputDialog(this, JCards.getMessage("new.counter.name"), JCards.getMessage("new.counter"), JOptionPane.QUESTION_MESSAGE);
		if (newCounter != null)
		{
			newCounter = newCounter.trim();
			if (newCounter.length() > 0)
			{
				Counter c = new Counter(newCounter, 0, 1);
				DBDef.getInstance().addCounter(c);
				countersList.setSelectedValue(newCounter, true);
			}
		}
		log.exiting("CounterPanel", "newItem");
	}
	
	/** Delete a counter */
	@Override
	public boolean deleteItem()
	{
		log.entering("CounterPanel", "deleteItem");
		boolean deleted = false;
		
		String counterName = (String)countersList.getSelectedValue();
		if (counterName != null)
		{
			String fName = DBDef.getInstance().removeCounter(counterName);
			if (fName != null)
			{
				JOptionPane.showMessageDialog(dbDialog,
						JCards.getFormattedMessage("counter.in.use", fName),
						JCards.getMessage("cannot.delete"), JOptionPane.ERROR_MESSAGE);
				deleted = true;
			}
		}
		log.exiting("CounterPanel", "deleteItem", deleted);
		
		return deleted;
	}

	@Override
	public boolean isItemSelected()
	{
		return !countersList.isSelectionEmpty();
	}
	
	
	/** Remove the current selection */
	@Override
	public void clearSelection()
	{
		countersList.clearSelection();
	}

	@Override
	public void cleanup()
	{
		DBDef.getInstance().deleteObserver(counterListModel);
	}
}
