/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import nrs.jcards.DBDef;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;
import nrs.jcards.gui.dialog.DBDialog;

/**
 * Edit category name and items
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class EditCategoryPanel extends JPanel implements AncestorListener
{
	private String catName;
	private DBDialog dbDialog;
	private JList catItemList;
	private JTextField catNameTF;
	private CategoryItemsListModel cilm;

	// listen for non-accelerator key presses
	private class KeyboardActionListener extends KeyAdapter
	{
		@Override
		public void keyPressed(KeyEvent e)
		{
			switch (e.getKeyCode())
			{
				case KeyEvent.VK_INSERT:
					addItem();
					break;

				case KeyEvent.VK_DELETE:
					deleteItem();
					break;

				case KeyEvent.VK_F2:
					renameItem();
					break;
			}
		}
	}

	/** Creates a new instance of EditCategoryPanel */
	public EditCategoryPanel(String catName, DBDialog dbDialog)
	{
		this.catName = catName;
		this.dbDialog = dbDialog;
		
		setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
				JCards.getLabelString("edit.category"), TitledBorder.CENTER, TitledBorder.ABOVE_TOP));
		
		cilm = new CategoryItemsListModel(catName);
		catItemList = new JList(cilm);
		catItemList.addKeyListener(new KeyboardActionListener());
		JScrollPane catScroll = new JScrollPane(catItemList);
		addAncestorListener(this);
		
		// right toolbar
		JToolBar rToolBar = new JToolBar(JToolBar.VERTICAL);
		rToolBar.setRollover(true);
		rToolBar.setFloatable(false);
		rToolBar.setBorderPainted(false);
		
		JButton upButton = new JCardsButton("cat.move.up", JCardsFrame.getImage(UserSettings.ICON_UP));
		upButton.setText(null);
		upButton.setMnemonic(KeyEvent.VK_UP);
		upButton.addActionListener(new ActionListener()
		{
			@Override public void actionPerformed(ActionEvent e)
			{
				moveItemUp();
			}
		});
		rToolBar.add(upButton);
		
		JButton downButton = new JCardsButton("cat.move.down", JCardsFrame.getImage(UserSettings.ICON_DOWN));
		downButton.setText(null);
		downButton.setMnemonic(KeyEvent.VK_DOWN);
		downButton.addActionListener(new ActionListener()
		{
			@Override public void actionPerformed(ActionEvent e)
			{
				moveItemDown();
			}
		});
		rToolBar.add(downButton);
		
		JButton addButton = new JCardsButton("cat.add", JCardsFrame.getImage(UserSettings.ICON_ADD));
		addButton.setText(null);
		addButton.addActionListener(new ActionListener()
		{
			@Override public void actionPerformed(ActionEvent e)
			{
				addItem();
			}
		});
		rToolBar.add(addButton);
		
		JButton delButton = new JCardsButton("cat.del", JCardsFrame.getImage(UserSettings.ICON_DELETE));
		delButton.setText(null);
		delButton.addActionListener(new ActionListener()
		{
			@Override public void actionPerformed(ActionEvent e)
			{
				deleteItem();
			}
		});
		rToolBar.add(delButton);
		
		JButton renameButton = new JCardsButton("cat.rename", JCardsFrame.getImage(UserSettings.ICON_EDIT));
		renameButton.setText(null);
		renameButton.addActionListener(new ActionListener()
		{
			@Override public void actionPerformed(ActionEvent e)
			{
				renameItem();
			}
		});
		rToolBar.add(renameButton);
		
		JButton sortAscButton = new JCardsButton("sort.asc", JCardsFrame.getImage(UserSettings.ICON_SORT_ASC));
		sortAscButton.setText(null);
		sortAscButton.addActionListener(new ActionListener()
		{
			@Override public void actionPerformed(ActionEvent e)
			{
				cilm.sortAscending();
				catItemList.requestFocusInWindow();
			}
		});
		rToolBar.add(sortAscButton);
		
		JButton sortDscButton = new JCardsButton("sort.dsc", JCardsFrame.getImage(UserSettings.ICON_SORT_DSC));
		sortDscButton.setText(null);
		sortDscButton.addActionListener(new ActionListener()
		{
			@Override public void actionPerformed(ActionEvent e)
			{
				cilm.sortDescending();
				catItemList.requestFocusInWindow();
			}
		});
		rToolBar.add(sortDscButton);
		
		setLayout(new BorderLayout(5, 5));
		setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(),
				JCards.getLabelString("category.edit"), TitledBorder.CENTER, TitledBorder.ABOVE_TOP));
		
		add(rToolBar, BorderLayout.EAST);
		add(catScroll, BorderLayout.CENTER);
		add(createCatPanel(dbDialog), BorderLayout.NORTH);
	}
	
	public void focusCategoryName()
	{
		catNameTF.requestFocus();
	}
	
	/** Category edit panel */
	private JPanel createCatPanel(DBDialog dbDialog)
	{
		JPanel catPanel = new JPanel(new BorderLayout(10, 5));
		catPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		
		JLabel label = new JCardsLabel("category");
		label.setFont(UIManager.getFont("Label.font").deriveFont(Font.BOLD));
		catPanel.add(label, BorderLayout.WEST);
		
		catNameTF = new JTextField(catName, 10);
		label.setLabelFor(catNameTF);
		catPanel.add(catNameTF, BorderLayout.CENTER);
		
		JButton updateButton = new JCardsButton("update", JCardsFrame.getImage(UserSettings.ICON_SAVE));
		updateButton.setToolTipText(JCards.getButtonString("cat.update.tt"));
		updateButton.addActionListener(new ActionListener()
		{
			@Override public void actionPerformed(ActionEvent e)
			{
				updateCategoryName();
			}
		});
		catPanel.add(updateButton, BorderLayout.EAST);
		
		dbDialog.getRootPane().setDefaultButton(updateButton);
		
		return catPanel;
	}
	
	private void updateCategoryName()
	{
		if (catNameTF.getText() != null && catNameTF.getText().length() > 0)
		{
			if (DBDef.getInstance().renameCategory(catName, catNameTF.getText()))
			{
				JOptionPane.showMessageDialog(dbDialog, JCards.getMessage("category.updated"),
						JCards.getMessage("saved"), JOptionPane.INFORMATION_MESSAGE);
			}
			catName = catNameTF.getText();
		}
		catNameTF.requestFocusInWindow();
	}
	
	/** Delete an item from a category list */
	private void deleteItem()
	{
		int index = catItemList.getSelectedIndex();
		if (index > -1)
		{
			String item = (String)catItemList.getSelectedValue();
			DBDef.getInstance().removeCategoryItem(catName, item);
			if (index >= catItemList.getModel().getSize()) {
				index--;
			}
			if (index < catItemList.getModel().getSize()) {
				catItemList.setSelectedIndex(index);
			}
		}
		catItemList.requestFocusInWindow();
	}
	
	/** Add an item to a category list */
	private void addItem()
	{
		String item = JOptionPane.showInputDialog(dbDialog, JCards.getMessage("item.new"), JCards.getLabelString("item"), JOptionPane.QUESTION_MESSAGE);
		if (item != null)
		{
			DBDef.getInstance().addCategoryItem(catName, item);
			catItemList.setSelectedValue(item, true);
		}
		catItemList.requestFocusInWindow();
	}
	
	/** Rename a category item */
	private void renameItem()
	{
		if (catItemList.getSelectedIndex() > -1)
		{
			String oldItem = catItemList.getSelectedValue().toString();
			String newItem = (String)JOptionPane.showInputDialog(dbDialog,
					JCards.getFormattedMessage("item.rename", oldItem),
					JCards.getLabelString("item"), JOptionPane.QUESTION_MESSAGE, null, null, oldItem);
			if (newItem != null)
			{
				DBDef.getInstance().renameCategoryItem(catName, oldItem, newItem);
				catItemList.setSelectedValue(newItem, true);
			}
		}
		catItemList.requestFocusInWindow();
	}
	
	/** Move a category item up one place */
	private void moveItemUp()
	{
		int index = catItemList.getSelectedIndex();
		if (index > 0)
		{
			String item = (String)catItemList.getSelectedValue();
			DBDef.getInstance().moveCategoryItem(catName, item, -1);
			catItemList.setSelectedValue(item, true);
		}
		catItemList.requestFocusInWindow();
	}
	
	/** Move a category item down one place */
	private void moveItemDown()
	{
		int index = catItemList.getSelectedIndex();
		if (index > -1)
		{
			String item = (String)catItemList.getSelectedValue();
			DBDef.getInstance().moveCategoryItem(catName, item, 1);
			catItemList.setSelectedValue(item, true);
		}
		catItemList.requestFocusInWindow();
	}

	@Override
	public void ancestorAdded(AncestorEvent event)
	{}

	@Override
	public void ancestorMoved(AncestorEvent event)
	{}

	@Override
	public void ancestorRemoved(AncestorEvent event)
	{
		DBDef.getInstance().deleteObserver(cilm);
	}
}
