/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.awt.Font;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.UIManager;
import nrs.jcards.Counter;
import nrs.jcards.DBDef;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;
import nrs.jcards.gui.action.JCardsAction;
import nrs.jcards.gui.dialog.DBDialog;

/**
 * Panel to edit a Counter
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class EditCounterPanel extends JPanel
{
	private DBDialog dbDialog;
	private Counter counter;
	private Font labelFont;
	
	/** Creates new form EditCounterPanel */
	public EditCounterPanel(DBDialog dbDialog, Counter counter)
	{
		this.dbDialog = dbDialog;
		this.counter = counter;
		
		labelFont = UIManager.getFont("Label.font").deriveFont(Font.BOLD);
		initComponents();

		dbDialog.getRootPane().setDefaultButton(updateButton);
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jLabel1 = new javax.swing.JLabel();
        nameTF = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        valueTF = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        incrementTF = new javax.swing.JTextField();
        updateButton = new javax.swing.JButton();

        setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), JCards.getLabelString("edit.counter"), javax.swing.border.TitledBorder.CENTER, javax.swing.border.TitledBorder.ABOVE_TOP));
        setLayout(new java.awt.GridBagLayout());

        jLabel1.setDisplayedMnemonic(JCardsAction.getMnemonic(JCards.getLabelString("name", true)));
        jLabel1.setFont(labelFont);
        jLabel1.setLabelFor(nameTF);
        jLabel1.setText(JCards.getLabelString("name")+":");
        jLabel1.setDisplayedMnemonicIndex(JCardsAction.getMnemonicIndex(JCards.getLabelString("name", true)));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(20, 10, 0, 10);
        add(jLabel1, gridBagConstraints);

        nameTF.setColumns(10);
        nameTF.setText(counter.getName());
        counter.getName();
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(20, 0, 0, 10);
        add(nameTF, gridBagConstraints);

        jLabel2.setDisplayedMnemonic(JCardsAction.getMnemonic(JCards.getLabelString("value", true)));
        jLabel2.setFont(labelFont);
        jLabel2.setLabelFor(valueTF);
        jLabel2.setText(JCards.getLabelString("value")+":");
        jLabel2.setDisplayedMnemonicIndex(JCardsAction.getMnemonicIndex(JCards.getLabelString("value", true)));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 0, 10);
        add(jLabel2, gridBagConstraints);

        valueTF.setColumns(10);
        valueTF.setText(Double.toString(counter.getValue()));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 10);
        add(valueTF, gridBagConstraints);

        jLabel3.setDisplayedMnemonic(JCardsAction.getMnemonic(JCards.getLabelString("increment", true)));
        jLabel3.setFont(labelFont);
        jLabel3.setLabelFor(incrementTF);
        jLabel3.setText(JCards.getLabelString("increment")+":");
        jLabel3.setDisplayedMnemonicIndex(JCardsAction.getMnemonicIndex(JCards.getLabelString("increment", true)));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 0, 10);
        add(jLabel3, gridBagConstraints);

        incrementTF.setColumns(10);
        incrementTF.setText(Double.toString(counter.getIncrement()));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 10);
        add(incrementTF, gridBagConstraints);

        updateButton.setIcon(JCardsFrame.getImage(UserSettings.ICON_SAVE));
        updateButton.setMnemonic(JCardsAction.getMnemonic(JCards.getButtonString("update", true)));
        updateButton.setText(JCards.getButtonString("update"));
        updateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.insets = new java.awt.Insets(20, 20, 20, 20);
        add(updateButton, gridBagConstraints);
    }// </editor-fold>//GEN-END:initComponents

	/**
	 * User clicked Update button
	 * @param evt
	 */
	private void updateButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_updateButtonActionPerformed
		String name = nameTF.getText();
		String val = valueTF.getText();
		String incr = incrementTF.getText();
		
		// error checking
		if (val.length() == 0)
		{
			JOptionPane.showMessageDialog(dbDialog, JCards.getMessage("null.value"), JCards.getMessage("error"), JOptionPane.ERROR_MESSAGE);
			valueTF.requestFocus();
			return;
		}
		if (incr.length() == 0)
		{
			JOptionPane.showMessageDialog(dbDialog, JCards.getMessage("null.increment"), JCards.getMessage("error"), JOptionPane.ERROR_MESSAGE);
			incrementTF.requestFocus();
			return;
		}
		
		double vall, incrl;
		try
		{
			vall = Double.valueOf(val).doubleValue();
			incrl = Double.valueOf(incr).doubleValue();
		}
		catch (NumberFormatException nfex)
		{
			JOptionPane.showMessageDialog(dbDialog, JCards.getMessage("val.incr.num"), JCards.getMessage("error"), JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		if (incrl == 0.0)
		{
			JOptionPane.showMessageDialog(dbDialog, JCards.getMessage("zero.increment"), JCards.getMessage("error"), JOptionPane.ERROR_MESSAGE);
			incrementTF.requestFocus();
			return;
		}
		
		Counter c = new Counter(name, vall, incrl);
		DBDef.getInstance().updateCounter(counter.getName(), c);
		counter = c;
		
		JOptionPane.showMessageDialog(dbDialog, JCards.getMessage("counter.updated"),
				JCards.getMessage("saved"), JOptionPane.INFORMATION_MESSAGE);
	}//GEN-LAST:event_updateButtonActionPerformed
	
	/** Set the focus to the field name field */
	public void focusCounterName()
	{
		nameTF.requestFocus();
	}
	
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField incrementTF;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JTextField nameTF;
    private javax.swing.JButton updateButton;
    private javax.swing.JTextField valueTF;
    // End of variables declaration//GEN-END:variables
}
