/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.util.Observable;
import java.util.Observer;
import javax.swing.AbstractListModel;
import nrs.jcards.DBDef;
import nrs.jcards.DBDefUpdate;

/**
 * A list that is aware of field changes within DBDef
 * 
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class FieldsListModel extends AbstractListModel implements Observer
{
	private DBDef dbDef = DBDef.getInstance();
	
	public FieldsListModel()
	{}
	
	/**
	 * The underlying data structure has changed
	 */
	@Override public void update(Observable o, Object arg)
	{
		assert arg != null : "FieldsListModel update arg is null";
		
		DBDefUpdate update = (DBDefUpdate)arg;
		System.out.println(update.getEvent());
		switch (update.getEvent())
		{
			case FIELD_ADD:
				fireIntervalAdded(this, update.getNewSize()-1, update.getNewSize()-1);
				break;
				
			case FIELD_CHANGE:
				fireContentsChanged(this, update.getNewIndex(), update.getNewIndex());
				break;
				
			case FIELD_DELETE:
				fireIntervalRemoved(this, update.getOldIndex(), update.getOldIndex());
				break;
				
			case FIELD_MOVE:
				fireContentsChanged(this, update.getOldIndex(), update.getNewIndex());
				break;
		}
	}
	
	/** Get the element at the index */
	@Override public Object getElementAt(int index)
	{
		return dbDef.getFieldNames().get(index);
	}
	
	/** Return the number of items in the list */
	@Override public int getSize()
	{
		return dbDef.getFieldNames().size();
	}
}
