/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.util.Iterator;
import java.util.List;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JPanel;
import nrs.jcards.FindReplaceDetails;
import nrs.jcards.JCards;
import nrs.jcards.gui.action.JCardsAction;

/**
 * Find panel
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class FindPanel extends JPanel
{
	private boolean replace = false;
	
	/** Creates new form FindPanel */
	public FindPanel(List<String> fields)
	{
		initComponents();
		fieldsCB.addItem(null);
		
		// add field names to fieldsCB
		Iterator<String> it = fields.iterator();
		while (it.hasNext())
		{
			fieldsCB.addItem(it.next());
		}
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        searchLabel = new javax.swing.JLabel();
        searchTF = new javax.swing.JTextField();
        replaceLabel = new javax.swing.JLabel();
        replaceTF = new javax.swing.JTextField();
        fieldLabel = new javax.swing.JLabel();
        fieldsCB = new javax.swing.JComboBox();
        matchCaseLabel = new javax.swing.JLabel();
        matchCaseCB = new javax.swing.JCheckBox();
        wholeWordsLabel = new javax.swing.JLabel();
        wholeWordsCB = new javax.swing.JCheckBox();
        refindLabel = new javax.swing.JLabel();
        refindCB = new javax.swing.JCheckBox();

        setLayout(new java.awt.GridBagLayout());

        searchLabel.setDisplayedMnemonic(getMnemonic("search"));
        searchLabel.setLabelFor(searchTF);
        searchLabel.setText(JCards.getLabelString("search")+":");
        searchLabel.setDisplayedMnemonicIndex(getMnemonicIndex("search"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(searchLabel, gridBagConstraints);

        searchTF.setColumns(15);
        searchTF.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(searchTF, gridBagConstraints);

        replaceLabel.setDisplayedMnemonic(getMnemonic("replace"));
        replaceLabel.setLabelFor(replaceTF);
        replaceLabel.setText(JCards.getLabelString("replace")+":");
        replaceLabel.setDisplayedMnemonicIndex(getMnemonicIndex("replace"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(replaceLabel, gridBagConstraints);

        replaceTF.setColumns(15);
        replaceTF.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(replaceTF, gridBagConstraints);

        fieldLabel.setDisplayedMnemonic(getMnemonic("field"));
        fieldLabel.setLabelFor(fieldsCB);
        fieldLabel.setText(JCards.getLabelString("field")+":");
        fieldLabel.setDisplayedMnemonicIndex(getMnemonicIndex("field"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(fieldLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(fieldsCB, gridBagConstraints);

        matchCaseLabel.setDisplayedMnemonic(getMnemonic("match.case"));
        matchCaseLabel.setLabelFor(matchCaseCB);
        matchCaseLabel.setText(JCards.getLabelString("match.case")+":");
        matchCaseLabel.setDisplayedMnemonicIndex(getMnemonicIndex("match.case"));
        matchCaseLabel.setDoubleBuffered(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(matchCaseLabel, gridBagConstraints);

        matchCaseCB.setBorder(null);
        matchCaseCB.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(matchCaseCB, gridBagConstraints);

        wholeWordsLabel.setDisplayedMnemonic(getMnemonic("whole.words"));
        wholeWordsLabel.setLabelFor(wholeWordsCB);
        wholeWordsLabel.setText(JCards.getLabelString("whole.words")+":");
        wholeWordsLabel.setDisplayedMnemonicIndex(getMnemonicIndex("whole.words"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(wholeWordsLabel, gridBagConstraints);

        wholeWordsCB.setBorder(null);
        wholeWordsCB.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(wholeWordsCB, gridBagConstraints);

        refindLabel.setDisplayedMnemonic(getMnemonic("find.in.results"));
        refindLabel.setLabelFor(refindCB);
        refindLabel.setText(JCards.getLabelString("find.in.results")+":");
        refindLabel.setDisplayedMnemonicIndex(getMnemonicIndex("find.in.results"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(refindLabel, gridBagConstraints);

        refindCB.setBorder(null);
        refindCB.setMargin(new java.awt.Insets(0, 0, 0, 0));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 10);
        add(refindCB, gridBagConstraints);
    }// </editor-fold>//GEN-END:initComponents
	
	
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel fieldLabel;
    private javax.swing.JComboBox fieldsCB;
    private javax.swing.JCheckBox matchCaseCB;
    private javax.swing.JLabel matchCaseLabel;
    private javax.swing.JCheckBox refindCB;
    private javax.swing.JLabel refindLabel;
    private javax.swing.JLabel replaceLabel;
    private javax.swing.JTextField replaceTF;
    private javax.swing.JLabel searchLabel;
    private javax.swing.JTextField searchTF;
    private javax.swing.JCheckBox wholeWordsCB;
    private javax.swing.JLabel wholeWordsLabel;
    // End of variables declaration//GEN-END:variables
	
	/** Get the displayed mnemonic for the label */
	private char getMnemonic(String key)
	{
		return JCardsAction.getMnemonic(JCards.getLabelString(key, true));
	}
	
	private int getMnemonicIndex(String key)
	{
		return JCardsAction.getMnemonicIndex(JCards.getLabelString(key, true));
	}
	
	public void setFindReplaceDetails(FindReplaceDetails frDetails, boolean findSubset)
	{
		if (frDetails.field != null)
		{
			fieldsCB.setSelectedItem(frDetails.field);
		}
		
		if (!findSubset)
		{
			refindLabel.setVisible(false);
			refindCB.setVisible(false);
		}
		
		refindCB.setSelected(frDetails.findInResults);
		matchCaseCB.setSelected(frDetails.matchCase);
		replace = frDetails.replaceOp;
		if (frDetails.replaceOp)
		{
			replaceTF.setText(frDetails.replaceText);
			fieldsCB.removeItemAt(0); // user must select a field
		}
		else
		{
			replaceLabel.setVisible(false);
			replaceTF.setVisible(false);
		}
		searchTF.setText(frDetails.searchText);
		wholeWordsCB.setSelected(frDetails.wholeWords);
	}
	
	public FindReplaceDetails getFindReplaceDetails()
	{
		FindReplaceDetails frDetails = new FindReplaceDetails();
		frDetails.field = (String)fieldsCB.getSelectedItem();
		frDetails.findInResults = refindCB.isSelected();
		frDetails.matchCase = matchCaseCB.isSelected();
		frDetails.replaceText = replaceTF.getText();
		if (frDetails.replaceText.length() == 0)
		{
			frDetails.replaceText = null;
		}
		frDetails.replaceOp = replace;
		frDetails.searchText = searchTF.getText();
		if (frDetails.searchText.length() == 0)
		{
			frDetails.searchText = null;
		}
		frDetails.wholeWords = wholeWordsCB.isSelected();
		
		if (frDetails.searchText == null)
		{
			refindCB.setEnabled(false);
		}
		
		return frDetails;
	}
	
	/**
	 * Set text fields - only text fields can be acted upon for find/replace
	 * @param fields List of valid fields
	 */
	public void setTextFields(List<String> fields)
	{
		DefaultComboBoxModel<String> cbm = (DefaultComboBoxModel<String>)fieldsCB.getModel();
		String item;
		for (int i=cbm.getSize()-1; i>-1; i--)
		{
			item = cbm.getElementAt(i);
			if (!fields.contains(item))
			{
				cbm.removeElement(item);
			}
		}
	}
}
