/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.URL;
import java.util.logging.Logger;
import javax.swing.JLabel;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;
import nrs.jcards.gui.action.HelpAction;

/**
 * Help label for handling context sensitive help
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class HelpLabel extends JLabel
{
	private String fragment;
	private JCardsFrame jcardsFrame;
	
	private final static Logger log = Logger.getLogger(HelpLabel.class.getName());
	
	public HelpLabel(JCardsFrame jcardsFrame, String fragment)
	{
		super(JCardsFrame.getImage(UserSettings.ICON_HELP));
		
		this.fragment = fragment;
		this.jcardsFrame = jcardsFrame;
		
		setToolTipText(JCards.getLabelString("display.help"));
		
		addMouseListener(new MouseAdapter() {
			@Override public void mouseClicked(MouseEvent e)
			{
				showHelp();
			}
		});
	}
	
	private void showHelp()
	{
		log.entering("HelpLabel", "showHelp");
		
		URL helpURL = HelpAction.getHelpFile(fragment);
		jcardsFrame.displayURL(helpURL);
		
		log.exiting("HelpLabel", "showHelp");
	}
}
