/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

/**
 * Class representing an icon that has an associated key, locale specific name and graphic filename
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class I18NIcon
{
	/** ResourceBundle key */
	private String key;
	/** Locale specific name */
	private String name;
	/** Filename of the icon graphic */
	private String graphic;
	
	/** Creates a new instance of I18NIcon */
	public I18NIcon() {}

	/** Return name */
	@Override
	public String toString()
	{
		return name;
	}
	
	/** Two I18NIcons are equal if their keys are the same */
	@Override
	public boolean equals(Object obj)
	{
		boolean eq = false;
		if (obj instanceof I18NIcon)
		{
			I18NIcon icon = (I18NIcon)obj;
			if (icon.getKey().equals(key)) {
				eq = true;
			}
		}
		
		return eq;
	}

	@Override
	public int hashCode()
	{
		int hash = 3;
		hash = 17 * hash + (this.key != null ? this.key.hashCode() : 0);
		hash = 17 * hash + (this.name != null ? this.name.hashCode() : 0);
		return hash;
	}
	
	/** Get the key
	 * @return key
	 */
	public String getKey()
	{
		return key;
	}
	
	/** Set the key
	 * @param key new key
	 */
	public void setKey(String key)
	{
		this.key = key;
	}
	
	/** Get the graphic name
	 * @return graphic filename
	 */
	public String getGraphic()
	{
		return graphic;
	}
	
	/** Set the graphic filename
	 * @param graphic new graphic filename
	 */
	public void setGraphic(String graphic)
	{
		this.graphic = graphic;
	}
	
	/** Get visible name
	 * @return Value of property name.
	 */
	public String getName()
	{
		return name;
	}
	
	/** Set visible name
	 * @param name New value of property name.
	 */
	public void setName(String name)
	{
		this.name = name;
	}
}
