/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.text.DateFormat;
import java.util.Date;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.filechooser.FileFilter;
import nrs.jcards.JCards;
import nrs.jcards.io.JCAReader;

/**
 * Provides brief details about a JCA file
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class JCAPreviewPanel extends JPanel implements PropertyChangeListener
{
	private boolean saveMode;
	private DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
	private final static Logger log = Logger.getLogger(JCAPreviewPanel.class.getName());
	
	/**
	 * Creates new form JCAPreviewPanel
	 * @param saveMode if true panel is part of save dialog; if false part of open dialog
	 */
	public JCAPreviewPanel(boolean saveMode)
	{
		this.saveMode = saveMode;
		initComponents();
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        java.awt.GridBagConstraints gridBagConstraints;

        jLabel1 = new javax.swing.JLabel();
        version = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        createDate = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        modDate = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        encrypted = new javax.swing.JCheckBox();

        setLayout(new java.awt.GridBagLayout());

        setPreferredSize(new java.awt.Dimension(180, 80));
        jLabel1.setFont(new java.awt.Font("SansSerif", 1, 10));
        jLabel1.setText(JCards.getLabelString("version")+":");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
        add(jLabel1, gridBagConstraints);

        version.setFont(new java.awt.Font("SansSerif", 0, 10));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 10, 10);
        add(version, gridBagConstraints);

        jLabel3.setFont(new java.awt.Font("SansSerif", 1, 10));
        jLabel3.setText(JCards.getLabelString("created")+":");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 10);
        add(jLabel3, gridBagConstraints);

        createDate.setFont(new java.awt.Font("SansSerif", 0, 10));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 10);
        add(createDate, gridBagConstraints);

        jLabel5.setFont(new java.awt.Font("SansSerif", 1, 10));
        jLabel5.setText(JCards.getLabelString("modified")+":");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 10);
        add(jLabel5, gridBagConstraints);

        modDate.setFont(new java.awt.Font("SansSerif", 0, 10));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 10);
        add(modDate, gridBagConstraints);

        jLabel2.setFont(new java.awt.Font("SansSerif", 1, 10));
        jLabel2.setText(JCards.getLabelString("encrypted")+":");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 10);
        add(jLabel2, gridBagConstraints);

        encrypted.setFont(new java.awt.Font("SansSerif", 0, 10));
        encrypted.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        encrypted.setMargin(new java.awt.Insets(0, 0, 0, 0));
        encrypted.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                encryptedActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 10);
        add(encrypted, gridBagConstraints);

    }// </editor-fold>//GEN-END:initComponents

	/**	Attempt to change the encrypted check box */
	private void encryptedActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_encryptedActionPerformed
	{//GEN-HEADEREND:event_encryptedActionPerformed
		encrypted.setSelected(!encrypted.isSelected());
	}//GEN-LAST:event_encryptedActionPerformed
	
	// Reset all components
	private void reset()
	{
		version.setText(null);
		createDate.setText(null);
		modDate.setText(null);
		encrypted.setSelected(false);
	}
	
	public void setVersion(String ver)
	{
		version.setText(ver);
	}
	
	public void setCreateDate(Date cDate)
	{
		createDate.setText(df.format(cDate));
	}
	
	public void setModifiedDate(Date mDate)
	{
		modDate.setText(df.format(mDate));
	}
	
	public void setEncrypted(boolean enc)
	{
		encrypted.setSelected(enc);
	}
	
	/**
	 * This method gets called when a bound property is changed.
	 *
	 * @param evt A PropertyChangeEvent object describing the event source
	 *   	and the property that has changed.
	 */
	@Override
	public void propertyChange(PropertyChangeEvent evt)
	{
		log.entering("JCAPreviewPanel", "propertyChange", evt);
		
		String prop = evt.getPropertyName();
		if (JFileChooser.SELECTED_FILE_CHANGED_PROPERTY.equals(prop))
		{
			reset();
			File file = (File)evt.getNewValue();
			if (file == null)
			{
				return;
			}
			String fileName = file.getName();
			if (fileName.endsWith(".jca") || fileName.endsWith(".jc5"))
			{
				Map<String, String> props = JCAReader.readHeader(file);
				if (!props.isEmpty())
				{
					try
					{
						String fileModDate = Long.toString(file.lastModified());
						String create = (props.get(JCAReader.CREATED) == null ? fileModDate : props.get(JCAReader.CREATED));
						Date date = new Date(Long.parseLong(create));
						createDate.setText(df.format(date));

						String mod = (props.get(JCAReader.MODIFIED) == null ? fileModDate : props.get(JCAReader.MODIFIED));
						date = new Date(Long.parseLong(mod));
						modDate.setText(df.format(date));
					}
					catch (NumberFormatException nfex)
					{
						log.log(Level.SEVERE, "Incorrect date format in JCA header", nfex);
					}
					
					encrypted.setSelected(Boolean.parseBoolean(props.get(JCAReader.ENCRYPTED)));
					version.setText(props.get(JCAReader.VERSION));
				}
			}
		}
		else if (saveMode && JFileChooser.FILE_FILTER_CHANGED_PROPERTY.equals(prop))
		{
			JFileChooser chooser = (JFileChooser)evt.getSource();
			FileFilter filter = chooser.getFileFilter();
			String description = filter.getDescription();
			String versionS = JCards.VERSION5;
			if (description.equals(JCards.getLabelString("jcards.files")))
			{
				versionS = JCards.VERSION;
			}
			version.setText(versionS);
		}
		log.exiting("JCAPreviewPanel", "propertyChange");
	}
	
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel createDate;
    private javax.swing.JCheckBox encrypted;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel modDate;
    private javax.swing.JLabel version;
    // End of variables declaration//GEN-END:variables
	
}
