/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import javax.swing.Icon;
import javax.swing.JButton;
import nrs.jcards.JCards;
import nrs.jcards.gui.action.JCardsAction;

/**
 * JButton that can handle mnemonics
 * 
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class JCardsButton extends JButton
{
	public JCardsButton(String resourceBundleKey)
	{
		this(resourceBundleKey, null);
	}

	/**
	 * Create a new JButton with the text looked up from a resource bundle.
	 * Mnemonic is set automatically if an underscore is used in the button text.
	 * Tooltip is set automatically if there is a resource bundle key ending with .tt.
	 * Action command is set automatically to the resource bundle key.
	 * @param resourceBundleKey
	 * @param icon may be null
	 */
	public JCardsButton(String resourceBundleKey, Icon icon)
	{
		String text = JCards.getButtonString(resourceBundleKey, true);
		JCardsAction.setButtonProperties(text, this);
		
		setIcon(icon);

		// see if there's a tooltip
		String tt = JCards.getButtonString(resourceBundleKey+".tt");
		if (tt != null) {
			setToolTipText(tt);
		}

		setActionCommand(resourceBundleKey);
	}
}
