/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.TransferHandler;
import javax.swing.TransferHandler.TransferSupport;
import nrs.jcards.JCards;

/**
 * Support for drag and dropping files onto the table
 * 
 * @author Copyright Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
class JCardsDropHandler extends TransferHandler
{
	private JCards jcards;
	private final static Logger log = Logger.getLogger(JCardsDropHandler.class.getName());

	public JCardsDropHandler(JCards jcards)
	{
		this.jcards = jcards;
	}

	@Override
	public boolean canImport(TransferSupport support)
	{
		log.entering("JCardsDropHandler", "canImport");

		boolean canImport = false;
		if (support.isDrop() &&
			(support.isDataFlavorSupported(DataFlavor.stringFlavor) || support.isDataFlavorSupported(DataFlavor.javaFileListFlavor)))
		{
			canImport = true;
		}

		log.exiting("JCardsDropHandler", "canImport", canImport);
		return canImport;
	}

	@Override public boolean importData(TransferSupport support)
	{
		log.entering("JCardsDropHandler", "importData");

		boolean imported = false;
		if (canImport(support))
		{
			DataFlavor[] flavors = support.getTransferable().getTransferDataFlavors();
			try
			{
				for (DataFlavor flavor : flavors)
				{
					if (flavor.equals(DataFlavor.stringFlavor))
					{
						// fetch the data and bail if this fails
						try {
							String fileNameList = (String)support.getTransferable().getTransferData(DataFlavor.stringFlavor);
							String[] fileNames = fileNameList.split("\n");
							URL url = new URL(fileNames[0]);
							imported = jcards.openFile(url.getPath());
						} catch (MalformedURLException mfex) {
							log.log(Level.WARNING, "Invalid file url specified", mfex);
						}
					}
					else if (flavor.equals(DataFlavor.javaFileListFlavor))
					{
						@SuppressWarnings("unchecked")
						List<File> fileList = (List<File>)support.getTransferable().getTransferData(DataFlavor.javaFileListFlavor);
						// only handle the first
						imported = jcards.openFile(fileList.get(0).getAbsolutePath());
					}
				}
			} catch (UnsupportedFlavorException e) {
				log.log(Level.WARNING, "Unsupported data import flavor", e);
			} catch (IOException e) {
				log.log(Level.WARNING, "Cannot read data", e);
			}
		}

		log.exiting("JCardsDropHandler", "importData", imported);
		return imported;
	}
}
