/*
 * JCards
 * Copyright (C) 2002-2011 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import java.io.Serializable;
import javax.swing.Icon;
import javax.swing.JLabel;
import nrs.jcards.JCards;
import nrs.jcards.gui.action.JCardsAction;

/**
 * JLabel that handles I18N text lookup and mnemonics
 *
 * @author Copyright &#169; 2002-2011 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class JCardsLabel extends JLabel implements Serializable
{
	private static final String COLON = ":";
	private String key;
	private boolean displayColon = true;
	
//	public JCardsLabel()
//	{
//	}
	
	/**
	 * Creates a new instance of JCardsLabel. By default will add a colon to the
	 * end of the label. 
	 * @param key ResourceBundle key
	 * @see JCardsLabel(String, boolean)
	 */
	public JCardsLabel(String key)
	{
		this(key, true);
	}
	
    /**
	 * Creates a new instance of JCardsLabel
	 * @param key ResourceBundle key
	 * @param displayColon if true add a colon to the end
	 */
    public JCardsLabel(String key, boolean displayColon)
    {
		//super();
		
		this.key = key;
		this.displayColon = displayColon;

		setText(key);
    }
	
	/**
	 * Creates a new instance of JCardsLabel
	 * @param key ResourceBundle key
	 * @param displayColon if true add a colon to the end
	 * @param icon an icon to display
	 */
	public JCardsLabel(String key, boolean displayColon, Icon icon)
	{
		super(icon);
		
		this.key = key;
		this.displayColon = displayColon;
		
		setText(key);
	}
	
	public void setDisplayColon(boolean displayColon)
	{
		this.displayColon = displayColon;
	}
	
	public boolean isDisplayColon()
	{
		return displayColon;
	}

	/**
	 * Set the text based on the contents of a ResourceBundle
	 * @param key the key into the ResourceBundle
	 */
	@Override public final void setText(String key)
	{
		if (key != null && key.length() > 0)
		{
			String value = JCards.getLabelString(key, true);
			String text = JCardsAction.getTextWithoutMnemonic(value);
			if (displayColon) {
				text += COLON;
			}
			super.setText(text);
			int index = JCardsAction.getMnemonicIndex(value);
			if (index > -1)
			{
				setDisplayedMnemonic(JCardsAction.getMnemonic(value));
				setDisplayedMnemonicIndex(index);
			}
		}
	}
	
	/**
	 * Get the resource bundle key used to initialise the label
	 * @return key, may be null
	 */
	public String getKey()
	{
		return key;
	}
}
