/*
 * JCards
 * Copyright (C) 2002-2011 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui;

import javax.swing.Action;
import javax.swing.JMenuItem;
import nrs.jcards.JCards;
import nrs.jcards.gui.action.JCardsAction;

/**
 * Handles mnemonics
 *
 * @author Copyright &#169; 2002-2011 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class JCardsMenuItem extends JMenuItem
{
	/**
	 * Creates a new instance of JCardsMenuItem.
	 * The mnemonic will be automatically set if there is an underscore in the text.
	 * The action command will automatically be set to the resource bundle key.
	 * @param resourceBundleKey resource bundle key
	 */
	public JCardsMenuItem(String resourceBundleKey)
	{
		String text = JCards.getMenuString(resourceBundleKey);
		setText(JCardsAction.getTextWithoutMnemonic(text));
		int mnIndex = JCardsAction.getMnemonicIndex(text);
		if (mnIndex > -1)
		{
			setMnemonic(JCardsAction.getMnemonic(text));
			setDisplayedMnemonicIndex(mnIndex);
		}
		setActionCommand(resourceBundleKey);
	}
	
	/** Create a new JCardsMenuItem based on an action */
	public JCardsMenuItem(JCardsAction jcAction)
	{
		super(jcAction);
		setMnemonic(jcAction.getMnemonic());
		setDisplayedMnemonicIndex(jcAction.getMnemonicIndex());
	}
	
	/**
	 * Create a new JCardsMenuItem based on an action and some I18N text
	 * @param action
	 * @param resourceBundleKey
	 */
	public JCardsMenuItem(Action action, String resourceBundleKey)
	{
		super(action);
		String text = JCards.getMenuString(resourceBundleKey);
		setText(JCardsAction.getTextWithoutMnemonic(text));
		int mnIndex = JCardsAction.getMnemonicIndex(text);
		if (mnIndex > -1)
		{
			setMnemonic(JCardsAction.getMnemonic(text));
			setDisplayedMnemonicIndex(mnIndex);
		}
	}
}
