/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.action;

import nrs.jcards.UserSettings;

/**
 * Enumeration of Actions
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public enum ActionEnum
{
	// format - class, localisation key, icon
	ABOUT("AboutAction", "about"),
	CATEGORY_EXPORT("CategoryExportAction", "cat.export"),
	COMMENTS("CommentsAction", "comments", UserSettings.ICON_COMMENT),
	CONTEXT_HELP("ContextHelpAction", "context.help", UserSettings.ICON_HELP),
	COPY_CARD("CopyCardAction", "card.copy", UserSettings.ICON_COPY),
	COPY_CLIPBOARD("CopyClipboardAction", "copy.clip", UserSettings.ICON_CLIPBOARD),
	CSV_EXPORT("CSVExportAction", "csv.export"),
	CSV_IMPORT("CSVImportAction", "csv.import", UserSettings.ICON_IMPORT),
	DB_DIALOG("DBAction", "fields", UserSettings.ICON_DB),
	DB_STATS("DBStatisticsAction", "db.stats", UserSettings.ICON_STATS),
	DELETE_CARD("DeleteCardAction", "card.del", UserSettings.ICON_DELETE),
	EDIT_CARD("EditCardAction", "card.edit", UserSettings.ICON_EDIT),
	EXIT("ExitAction", "exit", UserSettings.ICON_EXIT),
	FIELD_STATS("FieldStatisticsAction", "field.stats", UserSettings.ICON_STATS_FIELD),
	FIND("FindAction", "find", UserSettings.ICON_FIND),
	FIND_REPLACE("ReplaceAction", "replace"),
	FIND_RESET("FindResetAction", "find.reset", UserSettings.ICON_RESET),
	HELP("HelpAction", "help", UserSettings.ICON_HELP),
	HTML_CARD("HTMLCardsAction", "html.card"),
	HTML_TABLE("HTMLTableAction", "html.table"),
	NEW("NewAction", "new", UserSettings.ICON_NEW),
	NEW_CARD("NewCardAction", "card.new", UserSettings.ICON_ADD),
	OPEN("OpenAction", "open", UserSettings.ICON_OPEN),
	OPEN_URL("OpenURLAction", "open.url", UserSettings.ICON_OPEN_URL),
	PASSWORD("PasswordAction", "pass.set", UserSettings.ICON_PASSWORD),
	PREFS("PreferencesAction", "prefs", UserSettings.ICON_PREFERENCES),
	PRINT("PrintAction", "print", UserSettings.ICON_PRINT),
	SAVE("SaveAction", "save", UserSettings.ICON_SAVE),
	SAVE_AS("SaveAsAction", "save.as", UserSettings.ICON_SAVE_AS),
	TOOLBAR("ToolbarAction", "toolbar"),
	TUTORIAL("TutorialAction", "tutorial"),
	UNDELETE("UndeleteCardAction", "card.undel", UserSettings.ICON_UNDELETE),
	VIEW_ALL("ViewAllAction", "all"),
	WELCOME("WelcomeAction", "welcome"),
	XML_EXPORT("XMLExportAction", "xml.export");
	
	private String className;
	private String resourceName;
	private String iconName;

	/**
	 * New ActionEnum
	 * @param className name of class extending Action
	 * @param resource resource bundle name
	 */
	ActionEnum(String className, String resource)
	{
		this.className = className;
		resourceName = resource;
	}

	/**
	 * New ActionEnum
	 * @param className name of class extending Action
	 * @param resource resource bundle name
	 * @param icon icon to be used in Action
	 */
	ActionEnum(String className, String resource, String icon)
	{
		this.className = className;
		resourceName = resource;
		iconName = icon;
	}

	public String getClassName()
	{
		return className;
	}

	public String getResourceName()
	{
		return resourceName;
	}

	public String getIconName()
	{
		return iconName;
	}
}
