/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.action;

import java.awt.event.ActionEvent;
import java.io.File;
import java.util.Map;
import java.util.logging.Level;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import nrs.jcards.JCards;
import nrs.jcards.io.FileUtils;
import nrs.jcards.io.HTMLExporter;
import nrs.jcards.io.JCAReader;
import nrs.jcards.io.JCAReader.SplitFilename;

/**
 * Export data into table in HTML
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class HTMLTableAction extends JCardsAction
{
    private final static String STYLESHEET = "/nrs/jcards/xml/htmltable.xsl";
	private static final String EXT = ".html";
    private File saveFile;
    
    /** Creates a new instance of HTMLTableAction */
    public HTMLTableAction(JCards jcards, ActionEnum action)
    {
        super(jcards, action);
		
		setEnabled(false);
    }
    
    /**
     * Invoked when an action occurs.
     */
    @Override public void actionPerformed(ActionEvent e)
    {
        if (jcards.getCards().size() == 0)
        {
            JOptionPane.showMessageDialog(jcards.getFrame(), JCards.getMessage("no.data.save"), JCards.getMessage("no.data"), JOptionPane.INFORMATION_MESSAGE);
            return;
        }
        
		JPanel accPanel = new JPanel();
		Map<String, JCheckBox> accComps = XMLExportAction.getAccessoriesPanel(accPanel, false);
		
		if (saveFile == null && jcards.getCardFile() != null)
		{
			SplitFilename sf = JCAReader.splitFilename(jcards.getCardFile().getAbsolutePath());
			saveFile = new File(sf.name+EXT);
		}
        saveFile = FileUtils.getFile(JCards.getLabelString("html.table"), jcards.getFrame(), saveFile, accPanel);
        if (saveFile == null)
        {
            return;
        }
        
        String title = "JCards";
        if (jcards.getCardFile() != null)
        {
            title = jcards.getCardFile().getName();
            // remove suffix
            if (title.lastIndexOf('.') != -1)
            {
                title = title.substring(0, title.lastIndexOf('.'));
            }
        }
        try
        {
            HTMLExporter.export(jcards.getCards(), saveFile, STYLESHEET, title,
				accComps.get(XMLExportAction.ONLY_VISIBLE_FIELDS).isSelected(),
				accComps.get(XMLExportAction.INCLUDE_COMMENTS).isSelected());
            jcards.getFrame().setStatus(JCards.getFormattedMessage("exported.to", saveFile.getAbsolutePath()));
			
			if (accComps.get(XMLExportAction.OPEN_IN_BROWSER).isSelected())
			{
				jcards.getFrame().displayURL(saveFile.toURI().toURL());
			}
        }
        catch (Exception exc)
        {
			log.log(Level.WARNING, "Cannot export file", exc);
            jcards.getFrame().showError(JCards.getMessage("error"),
                    JCards.getMessage("cannot.export.html"), "HTMLTableAction", exc, Level.WARNING);
        }
    }
}
