/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.action;

import java.awt.event.ActionEvent;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Level;
import javax.swing.JFileChooser;
import nrs.jcards.JCards;
import nrs.jcards.gui.dialog.URLDialog;

/**
 * Open a file from a URL
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class OpenURLAction extends JCardsAction
{
	// @todo remember last entered URL
	private String lastURL = null;

	/** Creates a new instance of OpenURLAction */
	public OpenURLAction(JCards jcards, ActionEnum action)
	{
		super(jcards, action);
	}

	/** User selected action */
	@Override public void actionPerformed(ActionEvent e)
	{
		URLDialog ud = new URLDialog(jcards.getFrame());
		int ret = ud.showURLDialog();
		if (ret == JFileChooser.APPROVE_OPTION)
		{
			String urlString = ud.getURL();
			if (urlString != null)
			{
				lastURL = urlString;
				if (ud.isUseProxy())
				{
					System.setProperty("proxySet", "true");
					System.setProperty("proxyHost", ud.getHttpProxy());
					System.setProperty("proxyPort", ud.getHttpPort());
					System.setProperty("ftpProxySet", "true");
					System.setProperty("ftpProxyHost", ud.getFtpProxy());
					System.setProperty("ftpProxyPort", ud.getFtpPort());
				}

				try
				{
					URL url = new URL(urlString);
					jcards.openFile(url);
				}
				catch (MalformedURLException mex)
				{
					log.log(Level.WARNING, "Bad URL entered", mex);
					jcards.getFrame().showError(JCards.getMessage("error.open"), JCards.getMessage("bad.url"), 
						"OpenURLAction.actionPerformed", mex, Level.WARNING);
				}
			}
		}
	}
}
