/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.action;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JOptionPane;
import nrs.jcards.DBDef;
import nrs.jcards.FindReplaceDetails;
import nrs.jcards.JCards;
import nrs.jcards.fields.FieldDef;
import nrs.jcards.fields.FieldType;
import nrs.jcards.gui.dialog.FindDialog;

/**
 * Show find/replace dialog
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class ReplaceAction extends JCardsAction
{
	private FindReplaceDetails frDetails = new FindReplaceDetails();

	/** Creates a new instance of ReplaceAction */
	public ReplaceAction(JCards jcards, ActionEnum action)
	{
		super(jcards, action);

		frDetails.replaceOp = true;
		setEnabled(false);
	}

	@Override
	public void actionPerformed(ActionEvent e)
	{
		DBDef dbDef = DBDef.getInstance();
		// check that there are some text fields to do find/replace on
		if (dbDef.areTextFieldsDefined())
		{
			List<String> fields = dbDef.getFieldNames();
			FieldDef field;
			List<String> textFields = new ArrayList<String>(fields.size());
			for (String fieldName : fields)
			{
				field = dbDef.getFieldDef(fieldName);
				if (field.getType() == FieldType.STRING || field.getType() == FieldType.MEMO)
				{
					textFields.add(fieldName);
				}
			}

			FindDialog findDialog = new FindDialog(jcards.getFrame(), frDetails, fields, jcards.getCards().isFindSubset());
			findDialog.setTextFields(textFields);
			findDialog.pack();
			findDialog.setLocationRelativeTo(jcards.getFrame());
			findDialog.setVisible(true);

			frDetails = findDialog.getDetails();

			// may need to refresh the table display
			if (jcards.hasChanged())
			{
				jcards.getFrame().refreshTable();
			}
		}
		else
		{
			JOptionPane.showMessageDialog(jcards.getFrame(), JCards.getMessage("not.text.field"),
				JCards.getMessage("error"), JOptionPane.ERROR_MESSAGE);
		}
	}
}
