/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.action;

import java.awt.event.ActionEvent;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Level;
import nrs.jcards.JCards;

/**
 * Show the JCards tutorial
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class TutorialAction extends JCardsAction
{
	private static final String TUTORIAL_FILE = "tutorial.html";
	private static final String TUTORIAL_URL = "http://www.nsydenham.net/java/JCards/v6/tutorial.html";

	private URL tutFile;
	
	/** Creates a new instance of TutorialAction */
	public TutorialAction(JCards jcards, ActionEnum action)
	{
		super(jcards, action);
	}

	/**
	 * Invoked when an action occurs.
	 */
	@Override
	public void actionPerformed(ActionEvent e)
	{
		if (tutFile == null)
		{
			// look through classpath and user dir for jcards directory
			// we can then guess tutorial.html is in /docs/help/
			int index;
			String path = null;
			for (String cp : HelpAction.getPaths())
			{
				index = cp.toLowerCase().indexOf("jcards");
				if (index > -1)
				{
					path = HelpAction.findHelpFile(cp, index, TUTORIAL_FILE);
					if (path != null)
					{
						break; // found it
					}
				}
			}

			try
			{
				if (path == null) {
					tutFile = new URL(TUTORIAL_URL);
				} else {
					tutFile = new File(path).toURI().toURL();
				}
			} catch (MalformedURLException mfex) {
				log.log(Level.SEVERE, "Cannot find tutorial URL", mfex);
			}
		}
		
		jcards.getFrame().setStatus(JCards.getMessage("show.tutorial"));
		jcards.getFrame().displayURL(tutFile);
	}
}
