/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.dialog;

import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.NumberFormat;
import java.util.Locale;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import nrs.jcards.JCards;

/**
 * About JCards dialog
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class AboutDialog extends BaseDialog
{
	private static final Logger log = Logger.getLogger(AboutDialog.class.getName());
	
	/** Creates new form AboutDialog2 */
	public AboutDialog(JFrame parent)
	{
		super(parent, true);
		initComponents();

		// set mnemonics to be the firt character of each tab title
		String title;
		char mnemonic;
		for (int tabIndex=0; tabIndex<aboutTabs.getTabCount(); tabIndex++)
		{
			title = aboutTabs.getTitleAt(tabIndex);
			mnemonic = title.charAt(0);
			aboutTabs.setMnemonicAt(tabIndex, Character.toUpperCase(mnemonic));
		}

		try
		{
			contribPane.setPage(JCards.class.getResource("/nrs/jcards/xml/contrib.html"));
			BufferedReader reader = new BufferedReader(new InputStreamReader(getClass().getResourceAsStream("/nrs/jcards/props/licence.txt")));
			String line;
			StringBuilder sb = new StringBuilder(18000);
			while ((line = reader.readLine()) != null)
			{
				sb.append(line);
				sb.append("\n");
			}
			reader.close();
			licenceText.setText(sb.toString());
			licenceText.setCaretPosition(0);
		}
		catch (IOException ioex)
		{
			log.log(Level.WARNING, "Cannot load resource", ioex);
		}
	}
	
	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        aboutTabs = new javax.swing.JTabbedPane();
        homePanel = new javax.swing.JPanel();
        javax.swing.JLabel jLabel7 = new javax.swing.JLabel();
        contribPanel = new javax.swing.JPanel();
        javax.swing.JScrollPane contribScroll = new javax.swing.JScrollPane();
        contribPane = new javax.swing.JTextPane();
        licencePanel = new javax.swing.JPanel();
        licenceScroll = new javax.swing.JScrollPane();
        licenceText = new javax.swing.JTextArea();
        detailsPanel = new javax.swing.JPanel();
        javax.swing.JLabel versionLabel = new javax.swing.JLabel();
        javax.swing.JLabel versionValue = new javax.swing.JLabel();
        javax.swing.JLabel osLabel = new javax.swing.JLabel();
        javax.swing.JLabel osValue = new javax.swing.JLabel();
        javax.swing.JLabel javaLabel = new javax.swing.JLabel();
        javax.swing.JLabel javaValue = new javax.swing.JLabel();
        javax.swing.JLabel javaHomeLabel = new javax.swing.JLabel();
        javaHomeValue = new javax.swing.JLabel();
        javax.swing.JLabel localeLabel = new javax.swing.JLabel();
        javax.swing.JLabel localeValue = new javax.swing.JLabel();
        javax.swing.JLabel homeDirLabel = new javax.swing.JLabel();
        javax.swing.JLabel homeDirValue = new javax.swing.JLabel();
        javax.swing.JLabel currentDirLabel = new javax.swing.JLabel();
        javax.swing.JLabel currentDirValue = new javax.swing.JLabel();
        javax.swing.JLabel userDirLabel = new javax.swing.JLabel();
        javax.swing.JLabel userDirValue = new javax.swing.JLabel();
        javax.swing.JLabel memLabel = new javax.swing.JLabel();
        memValue = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle(JCards.getLabelString("about"));
        setFont(new java.awt.Font("Verdana", 0, 11)); // NOI18N

        aboutTabs.setPreferredSize(new java.awt.Dimension(550, 350));

        homePanel.setBackground(new java.awt.Color(255, 255, 255));
        homePanel.setLayout(new java.awt.BorderLayout());

        jLabel7.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel7.setIcon(new javax.swing.ImageIcon(getClass().getResource("/pics/logo.png"))); // NOI18N
        homePanel.add(jLabel7, java.awt.BorderLayout.CENTER);

        aboutTabs.addTab(JCards.getLabelString("about"), homePanel);

        contribPanel.setLayout(new java.awt.BorderLayout());

        contribPane.setEditable(false);
        contribPane.setFont(getFont());
        contribScroll.setViewportView(contribPane);

        contribPanel.add(contribScroll, java.awt.BorderLayout.CENTER);

        aboutTabs.addTab(JCards.getLabelString("contrib"), contribPanel);

        licencePanel.setLayout(new java.awt.BorderLayout());

        licenceText.setColumns(20);
        licenceText.setEditable(false);
        licenceText.setRows(5);
        licenceScroll.setViewportView(licenceText);

        licencePanel.add(licenceScroll, java.awt.BorderLayout.CENTER);

        aboutTabs.addTab(JCards.getLabelString("licence"), licencePanel);

        detailsPanel.setLayout(new java.awt.GridBagLayout());

        versionLabel.setFont(getFont());
        versionLabel.setText(JCards.getLabelString("product.version"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(20, 20, 5, 20);
        detailsPanel.add(versionLabel, gridBagConstraints);

        versionValue.setFont(getFont());
        versionValue.setText(JCards.VERSION);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(20, 5, 5, 20);
        detailsPanel.add(versionValue, gridBagConstraints);

        osLabel.setFont(getFont());
        osLabel.setText(JCards.getLabelString("os"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 20, 5, 20);
        detailsPanel.add(osLabel, gridBagConstraints);

        osValue.setFont(getFont());
        osValue.setText(System.getProperty("os.name")+" "
            +System.getProperty("os.arch")+" "
            +System.getProperty("os.version"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 20);
        detailsPanel.add(osValue, gridBagConstraints);

        javaLabel.setFont(getFont());
        javaLabel.setText(JCards.getLabelString("java"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 20, 5, 20);
        detailsPanel.add(javaLabel, gridBagConstraints);

        javaValue.setFont(getFont());
        javaValue.setText(System.getProperty("java.vm.vendor")+" "+
            System.getProperty("java.vm.name")+" "+
            System.getProperty("java.vm.version"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 20);
        detailsPanel.add(javaValue, gridBagConstraints);

        javaHomeLabel.setFont(getFont());
        javaHomeLabel.setText(JCards.getLabelString("java.home"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 20, 5, 20);
        detailsPanel.add(javaHomeLabel, gridBagConstraints);

        javaHomeValue.setFont(getFont());
        javaHomeValue.setText(System.getProperty("java.home"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 20);
        detailsPanel.add(javaHomeValue, gridBagConstraints);

        localeLabel.setFont(getFont());
        localeLabel.setText(JCards.getLabelString("system.locale"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 20, 5, 20);
        detailsPanel.add(localeLabel, gridBagConstraints);

        localeValue.setFont(getFont());
        localeValue.setText(Locale.getDefault().getDisplayName());
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 20);
        detailsPanel.add(localeValue, gridBagConstraints);

        homeDirLabel.setFont(getFont());
        homeDirLabel.setText(JCards.getLabelString("home.dir"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 20, 5, 20);
        detailsPanel.add(homeDirLabel, gridBagConstraints);

        homeDirValue.setFont(getFont());
        homeDirValue.setText(System.getProperty("user.home"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 20);
        detailsPanel.add(homeDirValue, gridBagConstraints);

        currentDirLabel.setFont(getFont());
        currentDirLabel.setText(JCards.getLabelString("current.dir"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 20, 5, 20);
        detailsPanel.add(currentDirLabel, gridBagConstraints);

        currentDirValue.setFont(getFont());
        currentDirValue.setText(System.getProperty("user.dir"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 20);
        detailsPanel.add(currentDirValue, gridBagConstraints);

        userDirLabel.setFont(getFont());
        userDirLabel.setText(JCards.getLabelString("user.dir"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 20, 5, 20);
        detailsPanel.add(userDirLabel, gridBagConstraints);

        userDirValue.setFont(getFont());
        userDirValue.setText(System.getProperty("user.home"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 20);
        detailsPanel.add(userDirValue, gridBagConstraints);

        memLabel.setFont(getFont());
        memLabel.setText(JCards.getLabelString("memory"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.RELATIVE;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 20, 20, 20);
        detailsPanel.add(memLabel, gridBagConstraints);

        memValue.setFont(getFont());
        memValue.setToolTipText(JCards.getLabelString("click.gc"));
        freeMemory(null);
        memValue.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                freeMemory(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 20, 20);
        detailsPanel.add(memValue, gridBagConstraints);

        aboutTabs.addTab(JCards.getLabelString("details"), detailsPanel);

        getContentPane().add(aboutTabs, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents
	
	private void freeMemory(java.awt.event.MouseEvent evt)//GEN-FIRST:event_freeMemory
	{//GEN-HEADEREND:event_freeMemory
		System.gc();
		Runtime rt = Runtime.getRuntime();
		NumberFormat nf = NumberFormat.getNumberInstance();
		memValue.setText(nf.format(rt.freeMemory()) + " / " + nf.format(rt.totalMemory()) + " bytes");
	}//GEN-LAST:event_freeMemory
	
	@Override
	public Dimension getPreferredSize()
	{
		return new Dimension(550, 450);
	}
	
	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[])
	{
		java.awt.EventQueue.invokeLater(new Runnable()
		{
			@Override
			public void run()
			{
				new AboutDialog(new javax.swing.JFrame()).setVisible(true);
			}
		});
	}
	
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTabbedPane aboutTabs;
    private javax.swing.JTextPane contribPane;
    private javax.swing.JPanel contribPanel;
    private javax.swing.JPanel detailsPanel;
    private javax.swing.JPanel homePanel;
    private javax.swing.JLabel javaHomeValue;
    private javax.swing.JPanel licencePanel;
    private javax.swing.JScrollPane licenceScroll;
    private javax.swing.JTextArea licenceText;
    private javax.swing.JLabel memValue;
    // End of variables declaration//GEN-END:variables
	
}
