/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.dialog;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import nrs.jcards.Cards;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;
import nrs.jcards.gui.CategoryPanel;
import nrs.jcards.gui.CounterPanel;
import nrs.jcards.gui.DBDialogActionsI;
import nrs.jcards.gui.FieldsPanel;
import nrs.jcards.gui.JCardsButton;
import nrs.jcards.gui.JCardsFrame;
import nrs.jcards.gui.JCardsLabel;
import nrs.jcards.gui.action.JCardsAction;

/**
 * Dialog for editing the database structure
 * 
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class DBDialog extends BaseDialog implements ChangeListener
{
	private JTabbedPane tabs;
	private JSplitPane split;
	private FieldsPanel fieldsPanel;
	private CategoryPanel catPanel;
	private CounterPanel countPanel;
	private JPanel addHelpPanel;
	private static final int SPLIT_POS = 300;
	
	/** Creates a new instance of DBDialog */
	public DBDialog(JCardsFrame jcardsFrame, Cards cards, boolean showCategoriesTab)
	{
		super(jcardsFrame, true);
		
		setTitle(JCards.getLabelString("edit.fields"));
		
		// when user clicks close window decoration
		addWindowListener(new WindowAdapter()
		{
			@Override
			public void windowClosing(WindowEvent we)
			{
				closeWindow();
			}
		});
		
		// create tabbed pane
		tabs = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.SCROLL_TAB_LAYOUT);

		fieldsPanel = new FieldsPanel(this, cards);
		tabs.addTab("Fields", fieldsPanel);
		JCardsLabel label = new JCardsLabel("fields", false, JCardsFrame.getImage(UserSettings.ICON_FIELDS));
		tabs.setTabComponentAt(0, label);
		tabs.setMnemonicAt(0, JCardsAction.getMnemonic(JCards.getLabelString("fields", true)));
		
		catPanel = new CategoryPanel(this);
		tabs.addTab("Categories", catPanel);
		label = new JCardsLabel("categories", false, JCardsFrame.getImage(UserSettings.ICON_CATEGORIES));
		tabs.setTabComponentAt(1, label);
		tabs.setMnemonicAt(1, JCardsAction.getMnemonic(JCards.getLabelString("categories", true)));
		
		countPanel = new CounterPanel(this);
		tabs.addTab("Counters", countPanel);
		label = new JCardsLabel("counters", false, JCardsFrame.getImage(UserSettings.ICON_COUNTER));
		tabs.setTabComponentAt(2, label);
		tabs.setMnemonicAt(2, JCardsAction.getMnemonic(JCards.getLabelString("counters", true)));
		if (showCategoriesTab)
		{
			tabs.setSelectedIndex(1);
		}
		tabs.addChangeListener(this);
		tabs.setEnabled(!showCategoriesTab); // only editing categories
		
		JPanel leftPanel = new JPanel(new BorderLayout());
		leftPanel.add(tabs, BorderLayout.CENTER);
		leftPanel.add(createControlsToolbar(), BorderLayout.SOUTH);
		
		// create content panel
		addHelpPanel = new JPanel(new BorderLayout(20, 20));
		JTextArea ta = new JTextArea(JCards.getLabelString("db.help"), 2, 20);
		ta.setEditable(false);
		addHelpPanel.add(ta, BorderLayout.CENTER);
		
		// create layout
		split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, leftPanel, addHelpPanel);
		split.setDividerLocation(SPLIT_POS);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(split);
	}
	
	/** Create a toolbar with common actions */
	public JToolBar createControlsToolbar()
	{
		JToolBar toolbar = new JToolBar();
		toolbar.setLayout(new BoxLayout(toolbar, BoxLayout.X_AXIS));
		toolbar.setFloatable(false);
		toolbar.setRollover(true);
		toolbar.setBorderPainted(false);
		
		JButton newField = new JCardsButton("add", JCardsFrame.getImage(UserSettings.ICON_ADD));
		newField.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				DBDialogActionsI dbPanel = (DBDialogActionsI)tabs.getSelectedComponent();
				dbPanel.newItem();
			}
		});
		toolbar.add(Box.createHorizontalStrut(55));
		toolbar.add(newField);
		
		JButton deleteField = new JCardsButton("delete", JCardsFrame.getImage(UserSettings.ICON_DELETE));
		deleteField.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				deleteItem();
			}
		});
		toolbar.add(Box.createHorizontalStrut(5));
		toolbar.add(deleteField);
		
		JButton okButton = new JCardsButton("ok", JCardsFrame.getImage(UserSettings.ICON_OK));
		okButton.setActionCommand("OK");
		okButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				closeWindow();
			}
		});
		toolbar.add(Box.createHorizontalStrut(5));
		toolbar.add(okButton);
		toolbar.add(Box.createHorizontalStrut(55));
		
		return toolbar;
	}
	
	private void deleteItem()
	{
		DBDialogActionsI dbPanel = (DBDialogActionsI)tabs.getSelectedComponent();
		if (dbPanel.isItemSelected()) // only offer to delete if something is selected
		{
			int ret = JOptionPane.showConfirmDialog(this, JCards.getMessage("are.you.sure"), JCards.getMessage("confirm.delete"), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
			if (ret == JOptionPane.YES_OPTION)
			{
				if (dbPanel.deleteItem()) {
					stateChanged(null); // remove right hand context panel
				}
			}
		}
	}
	
	/** Set the content pane */
	public void setContentPanel(JComponent contentPanel)
	{
		split.setRightComponent(contentPanel);
	}
	
	/**
	 * Get the preferred size of the DBDialog
	 * @return preferred size
	 */
	@Override
	public Dimension getPreferredSize()
	{
		return new Dimension(800, 350);
	}
	
	/** User changed tab */
	@Override
	public void stateChanged(ChangeEvent e)
	{
		DBDialogActionsI dbPanel = (DBDialogActionsI)tabs.getSelectedComponent();
		dbPanel.clearSelection();
		split.setRightComponent(addHelpPanel);
//		split.invalidate();
		split.setDividerLocation(SPLIT_POS);
	}
	
	/** Close the window and clear up any DBDef listeners */
	public void closeWindow()
	{
		catPanel.cleanup();
		countPanel.cleanup();
		fieldsPanel.cleanup();
		dispose();
	}
}
