/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.dialog;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.WindowConstants;
import nrs.jcards.DBDef;
import nrs.jcards.FindReplaceDetails;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;
import nrs.jcards.fields.FieldDef;
import nrs.jcards.gui.FindPanel;
import nrs.jcards.gui.JCardsFrame;
import nrs.jcards.gui.action.ActionEnum;
import nrs.jcards.gui.action.CancelAction;
import nrs.jcards.gui.action.ContextHelpAction;

/**
 * Dialog used for searching
 * 
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class FindDialog extends BaseDialog implements ActionListener
{
	private JCardsFrame jcardsFrame;
	private FindPanel fPanel;

	/**
     * Creates a new instance of FindDialog
     * @param jcards owning Frame
     * @param frDetails FindReplaceDetails
     * @param fields list of fields
     * @param findSubset find in subset
     */
	public FindDialog(JCardsFrame jcardsFrame, FindReplaceDetails frDetails, List<String> fields, boolean findSubset)
	{
		super(jcardsFrame, true);

		this.jcardsFrame = jcardsFrame;

		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		String titleKey = (frDetails.replaceOp ? "replace" : "find");
		setTitle(JCards.getLabelString(titleKey));
		setResizable(false);

		getContentPane().setLayout(new BorderLayout(2, 5));

		// create the Find panel
		fPanel = new FindPanel(fields);
		fPanel.setFindReplaceDetails(frDetails, findSubset);

		// create the toolbar
		JPanel toolbar = new JPanel(new FlowLayout(FlowLayout.RIGHT));

		ContextHelpAction help = (ContextHelpAction)jcardsFrame.getAction(ActionEnum.CONTEXT_HELP);
		help.setFragment("search");
		JButton helpButton = new JButton(help);
		toolbar.add(helpButton);
		
		JButton cancel = new JButton(new CancelAction(this));
		toolbar.add(cancel);

		String buttonKey = (frDetails.replaceOp ? "replace" : "search");
		JButton search = new JButton(JCards.getButtonString(buttonKey), JCardsFrame.getImage(UserSettings.ICON_FIND));
		search.setActionCommand("search");
		search.addActionListener(this);
		toolbar.add(search);
		getRootPane().setDefaultButton(search);

		getContentPane().add(fPanel, BorderLayout.CENTER);
		getContentPane().add(toolbar, BorderLayout.SOUTH);
	}

	/** User clicked a button */
	@Override public void actionPerformed(ActionEvent actionEvent)
	{
		FindReplaceDetails frDetails = fPanel.getFindReplaceDetails();
		if (frDetails.searchText == null)
		{
			dispose();
			return;
		}

		// find/replace is only allowed for String and Memo fields
		if (frDetails.replaceOp && frDetails.field != null)
		{
			FieldDef fd = DBDef.getInstance().getFieldDef(frDetails.field);
			switch (fd.getType())
			{
				case STRING:
				case MEMO:
					break;

				default:
					JOptionPane.showMessageDialog(jcardsFrame, JCards.getMessage("not.text.field"),
						JCards.getMessage("error"), JOptionPane.ERROR_MESSAGE);
					return;
			}
		}

		// if no cards are found offer to search again
		if (jcardsFrame.findCards(frDetails) == 0)
		{
			if (JOptionPane.showConfirmDialog(jcardsFrame, JCards.getMessage("repeat.search"),
				JCards.getMessage("no.cards.found"), JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
			{
				return;
			}
		}

		dispose();
	}

	public FindReplaceDetails getDetails()
	{
		return fPanel.getFindReplaceDetails();
	}

	/** Set the valid text fields in the FindPanel
     * @see FindPanel#setTextFields(List<String>)
     */
	public void setTextFields(List<String> fields)
	{
		fPanel.setTextFields(fields);
	}
}
