/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.dialog;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.font.TextAttribute;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;
import javax.swing.Action;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import nrs.jcards.JCards;
import nrs.jcards.gui.JCardsLabel;
import nrs.jcards.gui.JCardsPanel;
import nrs.jcards.gui.action.ActionEnum;

/**
 * Welcome dialog
 * @todo option to open example DB from web-site
 * @author Copyright 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class WelcomeDialog extends BaseDialog implements ChangeListener
{
	private JCards jcards;
	private static final String ONLINE_EXAMPLES = "http://www.nsydenham.net/java/JCards/examples/";
	
	/** Creates a new instance of WelcomeDialog */
	public WelcomeDialog(JCards jcards)
	{
		super(jcards.getFrame(), JCards.getLabelString("jcards.welcome"), true);
		
		this.jcards = jcards;
		
		JCardsPanel jcardsPanel = new JCardsPanel();
		setContentPane(jcardsPanel);
		
		jcardsPanel.setLayout(new GridBagLayout());
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.insets = new Insets(20, 20, 40, 20);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.weightx = 1.0;
		
		JLabel label = new JCardsLabel("jcards.welcome", false);
		label.setFont(new Font("LucidaBright", Font.BOLD, 22));
		jcardsPanel.add(label, gbc);
		
		gbc.insets = new Insets(10, 100, 10, 50);
		Font labelFont = new Font("LucidaBright", Font.BOLD, 14);
		jcardsPanel.add(createOption("jcards.open", labelFont), gbc);
		jcardsPanel.add(createOption("jcards.example", labelFont), gbc);
		jcardsPanel.add(createOption("jcards.online.examples", labelFont), gbc);
		jcardsPanel.add(createOption("jcards.tutorial", labelFont), gbc);
		jcardsPanel.add(createOption("jcards.help", labelFont), gbc);
		
		JCheckBox cb = new JCheckBox(JCards.getLabelString("jcards.show"));
		cb.setFont(new Font("Dialog", Font.PLAIN, 10));
		cb.setHorizontalTextPosition(SwingConstants.LEFT);
		cb.setOpaque(false);
		cb.setBorder(null);
		cb.setForeground(Color.BLUE);
		cb.setSelected(JCards.getUserSettings().isShowWelcome());
		cb.addChangeListener(this);
		gbc.anchor = GridBagConstraints.SOUTHEAST;
		gbc.weighty = 1.0;
		gbc.insets = new Insets(20, 20, 20, 5);
		jcardsPanel.add(cb, gbc);
		
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	}
	
	private JLabel createOption(String labelKey, Font labelFont)
	{
		JLabel label = new JCardsLabel(labelKey, false);
		label.setForeground(Color.BLUE);
		label.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		label.setFont(labelFont);
		label.setName(labelKey);
		label.addMouseListener(new MouseAdapter()
		{
			@Override public void mouseClicked(MouseEvent e)
			{
				JLabel ml = (JLabel)e.getSource();
				labelClicked(ml.getName());
			}

			@Override public void mouseEntered(MouseEvent e)
			{
				JLabel ml = (JLabel)e.getSource();
				Font f = ml.getFont();
				Map<TextAttribute, Object> attrs = new HashMap<TextAttribute, Object>();
				attrs.putAll(f.getAttributes());
				attrs.put(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON);
				ml.setFont(new Font(attrs));
				ml.setForeground(Color.RED);
			}

			@Override public void mouseExited(MouseEvent e)
			{
				JLabel ml = (JLabel)e.getSource();
				Font f = ml.getFont();
				Map<TextAttribute, Object> attrs = new HashMap<TextAttribute, Object>();
				attrs.putAll(f.getAttributes());
				attrs.remove(TextAttribute.UNDERLINE);
				ml.setFont(new Font(attrs));
				ml.setForeground(Color.BLUE);
			}
		});
		
		return label;
	}
	
    //	@Override
    //	public Dimension getPreferredSize()
    //	{
    //		return new Dimension(400, 350);
    //	}
	
	@Override
	public boolean isResizable()
	{
		return true;
	}
	
	/**
	 * Invoked when the target of the listener has changed its state.
	 *
	 * @param e  a ChangeEvent object
	 */
	@Override
	public void stateChanged(ChangeEvent e)
	{
		JCheckBox cb = (JCheckBox)e.getSource();
		JCards.getUserSettings().setShowWelcome(cb.isSelected());
	}
	
	private void labelClicked(String labelName)
	{
		if (labelName.equals("jcards.open"))
		{
			jcards.openFile((String)null);
		}
		else if (labelName.equals("jcards.example"))
		{
			jcards.createExampleDB();
		}
		else if (labelName.equals("jcards.tutorial"))
		{
			Action action = jcards.getFrame().getAction(ActionEnum.TUTORIAL);
			action.actionPerformed(null);
		}
		else if (labelName.equals("jcards.help"))
		{
			Action action = jcards.getFrame().getAction(ActionEnum.HELP);
			action.actionPerformed(null);
		}
		else if (labelName.equals("jcards.online.examples"))
		{
			try
			{
				URL url = new URL(ONLINE_EXAMPLES);
				jcards.getFrame().displayURL(url);
			} catch (MalformedURLException mfex) {
				Logger log = Logger.getLogger(WelcomeDialog.class.getName());
			}
		}
		
		dispose();
	}
}
