/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.table;

import java.awt.Component;
import java.util.List;
import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JTable;
import nrs.jcards.DBDef;
import nrs.jcards.fields.CategoryFieldDef;

/**
 * Table cell editor for category. User entered text isn't supported as Swing doesn't
 * distinguish between entered and cancelled text.
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CategoryEditor extends DefaultCellEditor
{
	/** Creates a new instance of CategoryRenderer */
	public CategoryEditor()
	{
		super(new JComboBox());
		JComboBox combo = (JComboBox)editorComponent;
		combo.setEditable(true);
	}

	@Override public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column)
	{
		DBDef dbDef = DBDef.getInstance();
		String fieldName = dbDef.getVisibleFields().get(column);
		CategoryFieldDef cfd = (CategoryFieldDef) dbDef.getFieldDef(fieldName);
		assert cfd != null : "getTableCellEditorComponent: Cannot get category for " + fieldName;
		List<String> catItems = DBDef.getInstance().getCategory(cfd.getCategory());
		assert catItems != null : "getTableCellEditorComponent: Cannot get category items for " + cfd.getCategory();
		
		JComboBox combo = (JComboBox)editorComponent;
		combo.removeAllItems();
		if (!cfd.isMandatory()) {
			combo.addItem(null); // default no selection
		}
		for (String item : catItems)
		{
			if (item != null && item.length() > 0)
			{
				combo.addItem(item);
			}
		}
		combo.setSelectedItem(value);

		return combo;
	}
}
