/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.table;

import java.awt.Component;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import nrs.jcards.DBDef;
import nrs.jcards.fields.DateFieldDef;

/**
 * Date format renderer for table
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class DateRenderer extends DefaultTableCellRenderer
{
	/** Creates a new instance of DateRenderer */
	public DateRenderer()
	{
	}

	/**
	 * Returns a table cell renderer for dates.
	 *
	 * @param table  the <code>JTable</code>
	 * @param value  the value to assign to the cell at
	 * 			<code>[row, column]</code>
	 * @param isSelected true if cell is selected
	 * @param hasFocus true if cell has focus
	 * @param row  the row of the cell to render
	 * @param column the column of the cell to render
	 * @return the default table cell renderer
	 */
	@Override
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
	{
		JLabel label = (JLabel)super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);

		// format the value
		if (value != null)
		{
			DBDef dbDef = DBDef.getInstance();
			String fieldName = dbDef.getVisibleFields().get(column);
			assert fieldName != null : "Cannot get field name for column "+column;
			DateFieldDef dfd = (DateFieldDef)dbDef.getFieldDef(fieldName);
			dfd.setGUIValue(label, value, false, false);
		}

		return label;
	}
}
