/*
 * JCards
 * Copyright (C) 2002-2011 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.gui.table;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;
import nrs.jcards.DBDef;
import nrs.jcards.JCards;
import nrs.jcards.UserSettings;
import nrs.jcards.gui.JCardsFrame;
import nrs.jcards.gui.JCardsMenuItem;

/**
 * Shows a popup menu when right-click on column header
 *
 * @author Copyright &#169; 2002-2011 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class TableColumnMouseListener extends MouseAdapter implements ActionListener
{
	private JCards jcards;
	private JPopupMenu popupMenu;
	private JMenuItem leftMI,  rightMI,  startMI,  endMI;
	private int column;

	/** Creates a new instance of TableColumnMouseListener */
	public TableColumnMouseListener(JCards jcards)
	{
		this.jcards = jcards;
		popupMenu = new JPopupMenu();

		startMI = createMenuItem("move.start", UserSettings.ICON_START);
		popupMenu.add(startMI);

		leftMI = createMenuItem("move.left", UserSettings.ICON_LEFT);
		popupMenu.add(leftMI);

		rightMI = createMenuItem("move.right", UserSettings.ICON_RIGHT);
		popupMenu.add(rightMI);

		endMI = createMenuItem("move.end", UserSettings.ICON_END);
		popupMenu.add(endMI);

		popupMenu.addSeparator();

		JMenuItem removeMI = createMenuItem("remove.col", UserSettings.ICON_DELETE);
		popupMenu.add(removeMI);
	}

	private JMenuItem createMenuItem(String menuProperty, String iconName)
	{
		JMenuItem mi = new JCardsMenuItem(menuProperty);
		if (iconName != null)
		{
			mi.setIcon(JCardsFrame.getImage(iconName));
		}
		mi.addActionListener(this);
		return mi;
	}

	/** Determine if popup should be shown */
	public void maybeShowPopup(MouseEvent e)
	{
		if (e.isPopupTrigger())
		{
			JTableHeader h = (JTableHeader) e.getSource();
			TableColumnModel columnModel = h.getColumnModel();
			int viewColumn = columnModel.getColumnIndexAtX(e.getX());
			column = columnModel.getColumn(viewColumn).getModelIndex();
			if (column != -1)
			{
				startMI.setVisible(true);
				leftMI.setVisible(true);
				rightMI.setVisible(true);
				endMI.setVisible(true);
				if (column == 0)
				{
					startMI.setVisible(false);
					leftMI.setVisible(false);
				}
				if (column == (columnModel.getColumnCount() - 1))
				{
					rightMI.setVisible(false);
					endMI.setVisible(false);
				}
				popupMenu.show(h, e.getX(), e.getY());
			}
		}
		else if (e.getButton() == MouseEvent.BUTTON1)
		{
			jcards.setChanged();
		}
	}

	/** Cross platform way of detecting popup event */
	@Override public void mouseReleased(MouseEvent e)
	{
		maybeShowPopup(e);
	}

	/** Cross platform way of detecting popup event */
	@Override public void mousePressed(MouseEvent e)
	{
		maybeShowPopup(e);
	}

	/** Invoked when an action occurs */
	@Override public void actionPerformed(ActionEvent e)
	{
		String ac = e.getActionCommand();
		List<String> visFields = DBDef.getInstance().getVisibleFields();
		String vf = visFields.get(column);

		if (ac.equals("move.start"))
		{
			visFields.remove(column);
			visFields.add(0, vf);
		}
		else if (ac.equals("move.left"))
		{
			visFields.remove(column);
			visFields.add(column - 1, vf);
		}
		else if (ac.equals("move.right"))
		{
			visFields.remove(column);
			visFields.add(column + 1, vf);
		}
		else if (ac.equals("move.end"))
		{
			visFields.remove(column);
			visFields.add(vf);
		}
		else if (ac.equals("remove.col") && visFields.size() > 1)
		{
			visFields.remove(column);
		}
		DBDef.getInstance().setVisibleFields(visFields);
	}
}
