/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.io;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import nrs.jcards.Card;
import nrs.jcards.Cards;
import nrs.jcards.DBDef;
import nrs.jcards.JCards;
import nrs.jcards.fields.FieldDef;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Parse Cards
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class CardsParser extends DefaultHandler
{
	private StringBuilder sb = new StringBuilder();
	private DBDef dbDef = DBDef.getInstance();
	private Cards cards;
	private String fieldName;
	private Map<String, Object> values;
	private final static Logger log = Logger.getLogger(CardsParser.class.getName());
	
	/** Creates a new instance of CardsParser */
	public CardsParser(JCards jcards)
	{
		cards = jcards.getCards();
	}
	
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException
	{
		sb.append(ch, start, length);
	}
	
	@Override
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	{
		log.entering("CardsParser", "startElement");
		
		if (qName.equalsIgnoreCase(Elements.CARD))
		{
			values = new HashMap<String, Object>();
		}
		else if (qName.equalsIgnoreCase(Elements.FIELD))
		{
			fieldName = attributes.getValue(0);
		}
		sb.setLength(0);
		
		log.exiting("CardsParser", "startElement");
	}
	
	@Override
	public void endElement(String uri, String localName, String qName) throws SAXException
	{
		log.entering("CardsParser", "endElement");
		
		if (qName.equalsIgnoreCase(Elements.INFO))
		{
			cards.setInfo(sb.toString());
		}
		else if (qName.equalsIgnoreCase(Elements.FIELD))
		{
			FieldDef fd = dbDef.getFieldDef(fieldName);
			if (fd == null)
			{
				log.log(Level.SEVERE, "Cannot find FieldDef for {0}", fieldName);
				throw new SAXException("Cannot find FieldDef for "+fieldName);
			}
			Object objVal = fd.parseValue(sb.toString()); // check value is OK
			if (objVal != null)
			{
				values.put(fieldName, objVal);
			}
		}
		else if (qName.equalsIgnoreCase(Elements.CARD))
		{
			Card card = new Card(values);
			cards.addCardSilently(card);
		}
		
		log.exiting("CardsParser", "endElement");
	}
}
