/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.io;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Parse column widths
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class ColWidthsParser extends DefaultHandler
{
	private StringBuilder sb = new StringBuilder();
	private List<Integer> colWidths = new ArrayList<Integer>();
	private final static Logger log = Logger.getLogger(ColWidthsParser.class.getName());

	@Override
	public void characters(char[] ch, int start, int length) throws org.xml.sax.SAXException
	{
		sb.append(ch, start, length);
	}

	@Override
	public void startElement(String uri, String localName, String qName, org.xml.sax.Attributes attributes) throws org.xml.sax.SAXException
	{
		sb.setLength(0);
	}

	@Override
	public void endElement(String uri, String localName, String qName) throws SAXException
	{
		log.entering("ColWidthsParser", "endElement", qName);

		if (qName.equalsIgnoreCase(Elements.WIDTH))
		{
			try
			{
				int i = Integer.parseInt(sb.toString());
				// check to handle JCards v5 bug where negative column widths
				// are stored
				if (i > 0) {
					colWidths.add(i);
				}
			}
			catch (NumberFormatException nfex)
			{
				log.log(Level.WARNING, "Incorrect column width specified " + sb.toString(), nfex);
			}
		}

		log.exiting("ColWidthsParser", "endElement");
	}

	/** Get the column widths */
	public List<Integer> getColumnWidths()
	{
		return colWidths;
	}
}
