/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.io;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import nrs.jcards.DBDef;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Parse categories
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class DBDefCategoryParser extends DefaultHandler
{
	private DBDef dbDef = DBDef.getInstance();
	private String categoryName;
	private List<String> catItems;
	private StringBuilder sb = new StringBuilder();
	private final static Logger log = Logger.getLogger(DBDefCategoryParser.class.getName());

	@Override
	public void characters(char[] ch, int start, int length) throws SAXException
	{
		sb.append(ch, start, length);
	}

	@Override
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	{
		log.entering("DBDefCategoryParser", "startElement", qName);

		if (qName.equalsIgnoreCase(Elements.CATEGORY))
		{
			for (int i = 0; i < attributes.getLength(); i++)
			{
				if (attributes.getQName(i).equalsIgnoreCase(Elements.NAME))
				{
					categoryName = attributes.getValue(i);
					catItems = new ArrayList<String>();
				}
			}
		}

		sb.setLength(0);

		log.exiting("DBDefCategoryParser", "startElement");
	}

	@Override
	public void endElement(String uri, String localName, String qName) throws SAXException
	{
		log.entering("DBDefCategoryParser", "endElement", qName);

		if (qName.equalsIgnoreCase(Elements.CAT_ITEM))
		{
			catItems.add(sb.toString());
		}
		else if (qName.equalsIgnoreCase(Elements.CATEGORY))
		{
			dbDef.setCategory(categoryName, catItems);
		}

		log.exiting("DBDefCategoryParser", "endElement");
	}
}
