/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.io;

import java.util.logging.Level;
import java.util.logging.Logger;
import nrs.jcards.Counter;
import nrs.jcards.DBDef;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Parse counters
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class DBDefCountersParser extends DefaultHandler
{
	private DBDef dbDef = DBDef.getInstance();
	private final static Logger log = Logger.getLogger(DBDefCountersParser.class.getName());

	@Override
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	{
		log.entering("DBDefCountersParser", "startElement", qName);

		if (qName.equalsIgnoreCase(Elements.COUNTER))
		{
			String attrName;
			String name = null,
			 val = "0",
			 incr = "0";
			for (int i = 0; i < attributes.getLength(); i++)
			{
				attrName = attributes.getQName(i);
				if (attrName.equalsIgnoreCase(Elements.NAME))
				{
					name = attributes.getValue(i);
				}
				else if (attrName.equalsIgnoreCase(Elements.VALUE))
				{
					val = attributes.getValue(i);
				}
				else if (attrName.equalsIgnoreCase(Elements.INCREMENT))
				{
					incr = attributes.getValue(i);
				}
			}

			if (name == null)
			{
				log.severe("No name set for counter");
				throw new SAXException("No name set for counter");
			}

			try
			{
				double value = Double.parseDouble(val);
				double increment = Double.parseDouble(incr);
				Counter counter = new Counter(name, value, increment);
				dbDef.addCounter(counter);
			}
			catch (NumberFormatException nfex)
			{
				log.log(Level.WARNING, "startElement: bad double value or increment", nfex);
			}
		}

		log.exiting("DBDefCountersParser", "startElement");
	}
}
