/*
 * JCards
 * Copyright (C) 2002-2011 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.io;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.SortOrder;
import nrs.jcards.DBDef;
import nrs.jcards.gui.SortDirective;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * SAX Parser of DBDef file
 *
 * @author Copyright &#169; 2002-2011 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class DBDefSettingsParser extends DefaultHandler
{
	private DBDef dbDef = DBDef.getInstance();
	private StringBuilder sb = new StringBuilder();
	private String defaultDateFormat;
	private boolean defaultPrepopDate = false;
	private SortOrder sortOrder = SortOrder.UNSORTED;
	private SortDirective sortedColumn = null;
	private final static Logger log = Logger.getLogger(DBDefSettingsParser.class.getName());
	
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException
	{
		sb.append(ch, start, length);
	}
	
	@Override
	public void endElement(String uri, String localName, String qName) throws SAXException
	{
		log.entering("DBDefSettingsParser", "endElement", qName);
		
		if (qName.equalsIgnoreCase(Elements.DATE_FORMAT))
		{
			defaultDateFormat = sb.toString(); // old JCA format
		}
		else if (qName.equalsIgnoreCase(Elements.PREPOPULATE_DATE))
		{
			defaultPrepopDate = Boolean.parseBoolean(sb.toString()); // old JCA format
		}
		else if (qName.equalsIgnoreCase(Elements.DISPLAY_FIELD_NAMES))
		{
			dbDef.setDisplayFieldNames(Boolean.parseBoolean(sb.toString()));
		}
		else if (qName.equalsIgnoreCase(Elements.DISPLAY_COLONS))
		{
			dbDef.setDisplayColons(Boolean.parseBoolean(sb.toString()));
		}
		else if (qName.equalsIgnoreCase(Elements.SORT_FIELD))
		{
			log.log(Level.FINEST, "Sort field={0}", sb);
			sortedColumn = new SortDirective(sb.toString(), sortOrder);
		}
		else if (qName.equalsIgnoreCase(Elements.VIS_FIELD))
		{
			dbDef.addVisibleField(sb.toString(), false);
		}
		
		log.exiting("DBDefSettingsParser", "endElement");
	}
	
	@Override
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	{
		log.entering("DBDefSettingsParser", "startElement", qName);
		
		sortOrder = SortOrder.UNSORTED;
		sb.setLength(0);
		
		if (qName.equalsIgnoreCase(Elements.SORT_FIELD))
		{
			for (int i=0; i<attributes.getLength(); i++)
			{
				if (attributes.getQName(i).equalsIgnoreCase(Elements.SORT_DIR))
				{
					sortOrder = JCardsSortOrder.getValue(attributes.getValue(i));
				}
			}
		}
		
		log.exiting("DBDefSettingsParser", "startElement");
	}
	
	public String getDefaultDateFormat()
	{
		return defaultDateFormat;
	}
	
	public boolean isDefaultPrepopDate()
	{
		return defaultPrepopDate;
	}
	
	public SortDirective getSortedColumn()
	{
		return sortedColumn;
	}
}
