/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.io;

import java.io.File;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import nrs.jcards.JCards;
import nrs.jcards.io.filter.HTMLFileFilter;
import nrs.jcards.io.filter.XMLFileFilter;

/**
 * Generic utility methods for getting a file to save to
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class FileUtils
{
	private static final String DEFAULT_FILENAME = "jcards.html";
	
	/** Get the home dir plus either Documents or My Documents */
	public static File getHomeDir()
	{
		File homeDir = new File(System.getProperty("user.home"));
		
		File homeDocs = new File(homeDir, "Documents");
		if (homeDocs.exists()) {
			return homeDocs;
		}

		homeDocs = new File(homeDir, "My Documents");
		if (homeDocs.exists()) {
			return homeDocs;
		}
		
		return homeDir;
	}
	
	/**
	 * Utility method to get a filename
	 * @param title title of the file dialog
	 * @param parent owning parent frame
	 * @param saveFile existing file reference to use
	 * @param accessoryComp options to be displayed at the side, may be null
	 */
	public static File getFile(String title, JFrame parent, File saveFile, JComponent accessoryComp)
	{
		JFileChooser choose = new JFileChooser();
		choose.setDialogTitle(title);
		if (saveFile == null) {
			saveFile = new File(getHomeDir(), DEFAULT_FILENAME);
		}
		choose.setSelectedFile(saveFile);
		FileFilter htmlFilter = new HTMLFileFilter();
		choose.addChoosableFileFilter(htmlFilter);
		choose.addChoosableFileFilter(new XMLFileFilter());
		choose.setFileFilter(htmlFilter);
		if (accessoryComp != null) {
			choose.setAccessory(accessoryComp);
		}
		
		File newFile = null;
		if (choose.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION)
		{
			newFile = choose.getSelectedFile();
			if (newFile.exists())
			{
				if (JOptionPane.showConfirmDialog(parent,
						JCards.getMessage("overwrite.existing"),
						JCards.getFormattedMessage("overwrite", newFile.getName()),
						JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION)
				{
					newFile = null;
				}
			}
		}
		
		return newFile;
	}
}
