/*
 * JCards
 * Copyright (C) 2002-2009 Nick Sydenham <nsydenham@yahoo.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package nrs.jcards.io;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Date;
import java.util.logging.Logger;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import nrs.jcards.Cards;
import nrs.jcards.JCards;
import org.jdom.output.XMLOutputter;

/**
 * HTML export of Cards
 *
 * @author Copyright &#169; 2002-2009 Nick Sydenham &lt;nsydenham@yahoo.co.uk&gt;
 */
public class HTMLExporter
{
	public static void export(Cards cards, File outputFile, String stylesheetName, String title, boolean onlyVisibleFields, boolean comments)
		throws IOException, TransformerConfigurationException, TransformerException
	{
		File tempFile = File.createTempFile("htmloutput", "xml");
		BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(tempFile));
		XMLOutputter xmlOut = new XMLOutputter();
		// output prolog
		bos.write(JCAReader.PROLOG.getBytes(JCAReader.UTF8));
		cards.writeCardsToOutputStream(xmlOut, bos, true, onlyVisibleFields, comments);
		bos.close();
		xmlOut = null;

		// transform
		TransformerFactory factory = TransformerFactory.newInstance();
		Templates stylesheet = factory.newTemplates(
			new StreamSource(JCards.class.getResourceAsStream(stylesheetName)));
		Transformer processor = stylesheet.newTransformer();
		processor.setParameter("title", title);
		DateFormat df = DateFormat.getDateTimeInstance();
		processor.setParameter("date", df.format(new Date()));
		processor.transform(new StreamSource(tempFile), new StreamResult(outputFile));

		if (!tempFile.delete())
		{
			Logger log = Logger.getLogger(HTMLExporter.class.getName());
			log.warning("Cannot delete temporary file "+tempFile.getAbsolutePath());
		}
	}
}
